/***************************************************************************
                          archivefs.h  -  description
                             -------------------
    begin                : tue dec 23 2003
    copyright            : (C) 2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef _ARCHIVEFS_H_
#define _ARCHIVEFS_H_

#include "virtualarch.h"
#include "virtualfs.h"
#include "enums.h"

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Gwna klasa zarzdzajca obiektami klas specyficznych dla archiww.
 Tworzone s tutaj obiekty klas obsugujcych poszczeglne archiwizery
 oraz uruchamiane czynnoci wykonywane na archiwach, np. listowanie.
 Zawarta jest tutaj take obsuga akcji, ktre naley wykona przed
 rozpoczciem, albo po zakoczeniu danego polecenia dla archiwum.
*/
class ArchiveFS : public VirtualFS
{
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param inputURL - nazwa archiwum na ktrym naley wykona operacj,
	 @param listViewExt wskanik dla widok listy plikw (konieczny dla rodzica).\n
	 Tworzone s tutaj obiekty klas archiwizerw (na podstawie podanej nazwy)
	 oraz czone sygnay klasy VirtualArch ze slotami z biecej.
	 */
	ArchiveFS( const QString & currentItem, ListViewExt *listViewExt );

	/** Destruktor klasy.\n
	 Usuwany jest tutaj obiekt klasy VirtualArch.
	 */
	~ArchiveFS();

	/** Zwraca kod bdu dla ostatniej operacji na archiwum.\n
	 @return kod bdu.
	 */
	Error errorCode() const { return mVirtualArch->errorCode(); }

//	QString host() const { return QString("arc://"+mVirtualArch->archiveName()+"/"); }
	/** Zwraca nazw archiwum z absolutn ciek.\n
	 @return nazwa archiwum z absolutn ciek.
	 */
	QString host() const { return currentURL().left(currentURL().find(mVirtualArch->archiveName()))+mVirtualArch->archiveName()+"/"; }

	/** Zwraca biec ciek w archiwum.\n
	 @return bieca cieka.
	 */
	QString path() const { return mVirtualArch->currentPath(); }

	/** Zwraca nazw aktualnie przetwarzanego pliku z absolutn ciek.\n
	 @return nazwa aktualnie przetwarzanego pliku.
	 */
	QString nameOfProcessedFile() const {
		return mVirtualArch->nameOfProcessedFile();
	}

	/** Pobierane s tu wyniki operacji waenia, tj. waga caoci, ilo plikw
	 i katalogw \n
	 @param weight - zmienna, w ktrej zapisana bdzie waga,
	 @param files - zmienna do zapisania iloci plikw,
	 @param dirs - zmienna do zapisania iloci katalogw.
	 */
	void getWeighingStat( long long & weigh, uint & files, uint & dirs ) {
		mVirtualArch->getWeighingStat( weigh, files, dirs );
	}

	/** Pobiera informacj o pliku wczeniej zainicjowan w skadowej
	 @em mNewUrlInfo .\n
	 @param newUrlInfo - zmienna, w ktrej naley umieci informacj o pliku.\n
	 Metoda jest wykorzystywana przez @em FilesPanel::slotResultOperation() ,
	 w celu zapewnienia odpowiedniej obsugi widoku listy.
	 */
	void getNewUrlInfo( UrlInfoExt & newUrlInfo ) {
		newUrlInfo = mNewUrlInfo;
	}

	/** Zwraca informacj o tym czy w biecym katalogu mona zapisywa.\n
	 @return status zapisywalnoci, moe on przyj nastpujce wartoci:
	  @li ENABLEdw - mona zapisywa,
	  @li DISABLEdw - brak praw zapisu,
	  @li UNKNOWNdw - nie wiadomo.\n
	 Aktualnie zawsze zwracana jest warto UNKNOWNdw.
	 */
	DirWritable currentDirWritable();

	/** Zwraca informacje czy archiwum w podanym pliku jest obsugiwane.\n
	 @param fileName - nazwa pliku archiwum.
	 @return TRUE - archiwum jest obsugiwane, przy czym aktualnie wspierane s
	 nastpujce rodzaje archiww: 'tar', 'tar.gz', 'tar.bz2'.
	 */
	static bool archiveIsSupported( const QString & fileName );

	/** Zwraca nazw katalogu roboczego.\n
	 @return nazwa katalogu roboczego z absolutn ciek.\n
	 Katalog roboczy jest wykorzystywany min. jako miejsce rozpakowywania plikw
	 do podgldu.
	 */
	QString workDirectory() const { return mVirtualArch->workDirectory(); }

private:
	VirtualArch *mVirtualArch;
	UrlInfoExt mNewUrlInfo;

	bool mSkipNextErrorOccures;

protected:
	/** Metoda uruchamia procedur listowania archiwum dla podanego katalogu.\n
	 @param fullPathName - nazwa katalogu do wylistowania, przy czym podanie '/'
	 spowoduje wylistowanie na najwyszym poziomie.
	 */
	void readDir( const QString & fullFileName );

	/** Ustawia ciek do katalogu roboczego.\n
	 @param workDirectory - nowa cieka do katalogu roboczego.
	 */
	void setWorkDirectoryPath( const QString & workDirectory ) {
		mVirtualArch->setWorkDirectory( workDirectory );
	}
// public slots:
// 	void slotPause( bool stop );
// 	void slotReadFileToView( const QString & fileName, QByteArray & buffer );

private slots:
	/** Slot wywoywany po zakoczeniu wykonywania operacji na archiwum.\n
	 @param error - TRUE oznacza, e podczas wykonywania operacji wystpi bd.
	 */
	void slotDone( bool error );

	/** Metoda powoduje wysanie sygnau z informacj o zmianie statusu
	 w przetwarzaniu archiwum.\n
	 @param state - status operacji.\n
	 Wysyany jest sygna o nazwie @em signalSetInfo().
	 */
	void slotStateChanged( int state );

	/** Slot wywoywany przed rozpoczciem wykonywania polecenia na archiwum.\n
	 @param command - polecenie do wykonania na archiwum.
	 */
	void slotCommandStarted( int command );

	/** Slot wywoywany po zakoczeniu wykonywania polecenia na archiwum.\n
	 @param command - wykonane polecenie na archiwum,
	 @param error - TRUE oznacza, e podczas wykonywania polecenia wystpi bd,
	 przy czym warto bdu mona odczyta przy pomocy metody @em errorCode().
	 */
	void slotCommandFinished( int command, bool error );

//	void slotUpdateItemOnLV( const QString & );
};

#endif
