/***************************************************************************
                          listviewext.h  -  description
                             -------------------
    begin                : Mon Jul 9 2001
    copyright            : (C) 2001 by Mariusz Borowski (initializer)
                         : (C) 2002,2003 by Piotr Mierzwiski (developer)
    email                : mborowski@silesia.net, peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef LISTVIEWEXT_H
#define LISTVIEWEXT_H

#include <qheader.h>
#include <qlistview.h>
#include <qstringlist.h>

#include "listviewfinditem.h"
#include "listvieweditbox.h"
#include "listviewitemext.h"
#include "urlinfoext.h"
#include "popupmenu.h"
#include "filters.h"
#include "enums.h"

static const UrlInfoExt scNullUrlInfoExt;

/**
  *@authors: Mariusz Borowski (initialized), Piotr Mierzwiski (ext.and develop)
 **/
/** @short Klasa widoku listy plikw.
 Jest to rozszerzona wzgldem QListView klasa, przystosowana do obsugi listy
 plikw. Zawiera wiele pomocnych metod pomagajcych w obsudze listy plikw.
 Nale do nich:\n
  @li pokazywanie listy w dwch widokach: jako zwykej listy i jako drzewa;
  @li wiele metod kolorowania listy i jej elementw (kolorowanie elementw przy
   zaznaczeniu i bez niego, kolorowanie ta, kolorowanie kursora, rysowanie
	 siatki w okrelonym kolorze);
  @li ustawianie tzw. penego kursora lub kursora jako dwukolorowej ramki;
  @li ustawianie rodziny i rozmiaru fontu dla elementw listy;
  @li pokazywanie i chowanie ikon dla elementw (w pierwszej kolumnie), przy
   czym rodzaj ikony uzaleniony jest od rozszerzenia pliku;
  @li ustawianie sortowania dla okrelonej kolumny i w okrelonym kierunku;
  @li wczana/wyczana waciwo pokazywania cay czas kursora podczas
   przesuwania (w pionie) zawartoci widoku;
  @li wiele metod zaznaczania elementw (pojedynczo, grupami wg wzorca,
   wszystkie z okrelonym rozszerzeniem, w zakresie wskazanym mysz, wszystkie
   istniejce). Przy czym mona wybra do sortowania same pliki, same katalogi,
   lub jedne i drugie.
  @li wywietlanie wybranych kolumn (moliwo wyczania i wczania
   okrelonych);
  @li pokazywanie postpu wstawiania na list elementw
  @li moliwo filtrowania elementw listy. Mog by pokazywane:
   - wszystkie pliki i katalogi, oprcz ukrytych,\n
   - wszystkie pliki i katalogi,\n
   - tylko katalogi,\n
   - elementy pasujce do filtru zdefiniowanego przez uytkownika \n \n
 Tworzone s tu rwnie obiekty szybkiego wyszukiwania elementu na licie,
 zmiany nazwy, oraz menu podrcznego.
*/
class ListViewExt : public QListView
{
	friend class ListViewItemExt;
	Q_OBJECT
//private:
// 	typedef QValueList<UrlInfoExt> UrlInfoList;
// 	UrlInfoList mBufferUrlsList;

public:
	/** Konstruktor listy.\n
	 @param preParent - wskanik rodzica dla rodzica listy (inicjowany jest nim
	  obiekt klasy QListView),
	 @param parent - wskanik rodzica dla listy (uywany do wysyania zdarzenia),
	 @param popupMenu - TRUE oznacza, e naley utworzy obiekt menu podrcznego,
	 @param toolTip - TRUE oznacza, e naley utworzy obiekt dymka dla elementu,
	 @param renameBox - TRUE oznacza, e naley utworzy obiekt okienka
	  edycyjnego, sucego do szybkiej zmiany wartoci komrki biecego elementu
	 @param findDlg - TRUE oznacza, e naley utworzy obiekt dialogu do szybkiego
	  sucego do wyszukiwania elementu na licie,
	 @param name - nazwa dla obiektu.\n
	 Zawiera domylne ustawienia dla listy, czyli: brak siatki, szary kolor
	 siatki, lista bez dwu kolorego ta, drugi kolor ta dwukolorowego jako blady
	 pomaracz, biay kolor ta, granatowy wypeniony kursor, pokazuj ikony przy
	 elemencie w pierwszej kolumnie, czcionka na licie to Helvetica o rozmiarze
	 10, nie grupuj rozmiarw plikw.
	 */
	ListViewExt( QWidget *preParent, QWidget *parent, bool popupMenu=TRUE, bool toolTip=TRUE, bool renameBox=TRUE, bool findDlg=TRUE, const char *name=0 );

	/** Destruktor klasy.\n
	 Usuwane s tutaj obiekty tworzone wewntrz.
	 */
	~ListViewExt();


	/** Metoda ustawia kolor ta dla zaznaczonego tekstu na podany.\n
	 @param highlBgColor - nowy kolor ta.
	 */
	void setHighlightBgColor( const QColor & highlBgColor );

	/** Zwraca kolor ta dla zaznaczonego tekstu.\n
	 @return kolor ta.
	 */
	const QColor & highlightBgColor() const;


	/** Metoda ustawia kolor zaznaczonego tekstu na podany.\n
	 @param highlTextColor - kolor do ustawienia.
	 */
	void setHighlightedTextColor( const QColor & highlTextColor );

	/** Zwraca kolor kolor zaznaczonego tekstu.\n
	 @return kolor tekstu.
	 */
	const QColor & highlightedTextColor() const;


	/** Metoda ustawia kolor niezaznaczonego tekstu na podany.\n
	 @param txtColor - kolor do ustawienia
	 */
	void setTextColor( const QColor & txtColor );

	/** Zwraca kolor niezaznaczonego tekstu.\n
	 @return kolor tekstu.
	 */
	const QColor & textColor() const;


	/** Metoda ustawia kolor tekstu niezaznaczonego elementu pod kursorem.\n
	 @param c - nowy kolor tekstu.
	 */
	void setTextColorUnderCursor( const QColor & c );

	/** Zwraca kolor tekstu niezaznaczonego elementu pod kursorem.\n
	 @return kolor tekstu.
	 */
	const QColor & textColorUnderCursor() const;


	/** Metoda ustawia kolor tekstu zaznaczonego elementu pod kursorem.\n
	 @param c - nowy kolor tekstu.
	 */
	void setHighlightTextColorUnderCursor( const QColor & c );

	/** Zwraca kolor tekstu zaznaczonego elementu pod kursorem.\n
	 @return kolor tekstu.
	 */
	const QColor & highlightedTextColorUnderCursor() const;


	/** Metoda ustawia drugi kolor ta (to moe by w paski).\n
	 @param c - nowy kolor drugiego paska ta (pierwszy jest zawsze biay).
	 */
	void setSecondColorOfBg( const QColor & c );

	/** Zwraca drugi kolor ta.\n
	 @return drugi kolor ta.
	 */
	const QColor & secondColorOfBg() const;

	/** Wcza/wycza dwukolorowe to.\n
	 @param TRUE - wymusza rysowanie dwukolorowego to, w przeciwnym razie bdzie
	 mie jeden kolor.\n
	 Domylnie waciwo ta jest wyczona.
	 */
	void setEnableTwoColorsOfBg( bool enable = TRUE );

	/** Zwraca informacj o tym czy to jest dwukolorowe.\n
	 @return TRUE - oznacza, e to jest dwukolorowe, w przeciwnym razie ma jeden.
	 */
	bool enabledTwoColorsOfBg() const;


	/** Metoda ustawia kolor siatki (domylnym jest szary - 'Qt::gray').\n
	 @param gridColor - nowy kolor dla siatki.
	 */
	void setGridColor( const QColor & gridColor );

	/** Zwraca kolor siatki.\n
	 @return kolor siatki.
	 */
	const QColor & gridColor() const;

	/** Metoda wcza/wycza pokazywanie siatki.\n
	 @param showGrid - rwne TRUE, oznacza e siatka zostanie pokazana,
	 w przeciwnym razie nie jest bdzie rysowana
	 */
	void setShowGrid( bool showGrid = TRUE );

	/** Zwraca informacj o tym czy siatka jest pokazywana.\n
	 @return TRUE, jeli jest pokazywana, w przeciwnym razie FALSE.
	 */
	bool grid() const;


	/** Metoda wcza/wycza tzw. peny kursor (belka wypeniona jednym kolorem).\n
	 @param fullCursor - rwne TRUE, oznacza e rysowany bdzie peny kursor,
	 w przeciwnym razie rysowana jest dwukolorowa ramka.\n
	 Domylnie waciwoc ta jest wczona.
	 */
	void setFullCursor( bool fullCursor = TRUE );

	/** Metoda ustawia kolor dla tzw. penego kursora (belka wypeniona jednym
	 kolorem). Domylnym jest kolor granatowy ('Qt::darkBlue').\n
	 @param c - nowy kolor penego kursora.
	 */
	void setFullCursorColor( const QColor & c );

	/** Zwraca informacj o tym czy pokazywany jest tzw. peny kursor.\n
	 @return TRUE jeli pokazywany jest peny kursor, w przeciwnym razie FALSE.
	 */
	bool isFullCursor() const;


	/** Metoda wcza/wycza rysowanie kursora jako dwukolorowej ramki.\n
	 @param dblFrameCursor - rwne TRUE, wymusza rysowanie kursora jako
	 dwukolorowej ramki.\n
	 Domylnie waciwoc ta jest wyczona.
	 */
	void setDblFrameCursor( bool dblFrameCursor = TRUE );

	/** Metoda ustawia kolory dla kursora jako dwukolorowej ramki.\n
	 @param colInside - kolor dla wewntrznej ramki,
	 @param colOutside - kolor dla zewntrznej ramki.
	 */
	void setDblFrameCursorColors( const QColor & colInside, const QColor & colOutside );


	/** Metoda ustawia kolor dla kursora penego i dwukolorowej ramki.\n
	 @param cursorColor - nowy kolor kursora, przy czym dla kursora jako
	 dwukolorowej (obywdwie ramki bd miay ten sam kolor).
	 */
	void setCursorColor( const QColor & cursorColor );

	/** Zwraca kolor kursora.\n
	 @return kolor kursora.
	 */
	const QColor & cursorColor() const;

	/** Zwraca kolor wewntrznej ramki kursora.\n
	 @return kolor wewntrznej ramki.
	 */
	const QColor & frameInsCursorColor() const;

	/** Zwraca kolor zewntrznej ramki kursora.\n
	 @return kolor zewntrznej ramki.
	 */
	const QColor & frameOutCursorColor() const;


	/** Zwraca rozmiar biecego fontu listy (w punktach).\n
	 @return rozmiar fontu.
	 */
	uint fontSize() const { return mFontSize; }

	/** Zwraca nazw rodziny biecego fontu listy.
	 @return nazwa rodziny fontu.
	 */
	QString fontFamily() const { return mFontFamily; }

	/** Ustawia rozmiar biecego fontu (w punktach) na podany.\n
	 @param fontSize - rozmiar fontu.
	 */
	void setFontSize( const uint fontSize ) { setFontList( mFontFamily, fontSize ); }

	/** Ustawia rodzin fontu na podan.\n
	 @param fontFamily - nowa rodzina fontu.
	 */
	void setFontFamilly( const QString & fontFamily ) { setFontList( fontFamily, mFontSize ); }

	/** Metoda powoduje ustawienie rodziny i rozmiaru czcionki dla listy.\n
	 @param fontFamily - rodzina czcionki,
	 @param fontSize - rozmiar czcionki (w punktach).
	 */
	void setFontList( const QString & fontFamily, const uint fontSize );


	/** Funkcja uruchamia formatowanie podanego cigu cyfr.\n
	 @param numStr - cig do sformatowania i umieszczenia wyniku.\n
	 Wywoywana jest tu funkcja zewntrzna formatujca cig cyfr. Przy czym rodzaj
	 formatowania jest ustawiony w skadowej @em mKindOfFormating. Wynik
	 formatowania zapisywany jest w podanym parametrze.\n
	 */
	void formatNumberStr( QString & numStr );

	/** Wcza/wycza wywietlanie ikon (na pocztku pierwszej kolumny).\n
	 @param showIcons - rwne TRUE - ikony bd pokazywane, w przeciwnym razie
	 nie zostan wywietlone.\n
	 */
	void setShowIcons( bool showIcons = TRUE );


	/** Zwraca ikone odpowiadajc rozszerzeniu pliku podanego jako obiekt
	 informacji o pliku.\n
	 @param urlInfo - adres obiektu z informacj o pliku,
	 @return wskanik na nowoutworzony obiekt QPixmap.\n
	 Przy czym prawa do uruchamiania maj wikszy priorytet od rozszerzenia pliku.
	 */
	QPixmap * icon( const UrlInfoExt & ) const;


	/** Zwraca informacj o tym czy pokazywane s ikony.\n
	 @return TRUE - oznacza, e ikony s pokazywane, w przeciwnym wypadku s
	 niewidoczne.
	 */
	bool showIcons() const { return mShowIcons; }

	/** Zwraca szeroko ikony dla elementu listy.\n
	 @return szeroko ikony (w punktach).
	 */
	uint iconWidth() const { return mIconWidth; }


	/** Zwraca informacj o aktualnym rodzaju formatowania rozmiarw plikw.\n
	 @return rodzaj formatowania, patrz: @see KindOfFormating.
	 */
	KindOfFormating kindOfGroupFileSize() const { return mKindOfFormating; }

	/** Ustawia rodzaj formatowania rozmiarw plikw na podany.\n
	 @param kindOfFormating - rodzaj formatowania, patrz: @see KindOfFormating.
	 */
	void setKindOfGroupFileSize( KindOfFormating kindOfFormating ) { mKindOfFormating = kindOfFormating; }


	/** Metoda suy do zmiany widoku listy na podany.\n
	 @param kindOfView - nowy widok dla listy typu @see KindOfListView
	 */
	void setKindOfView( KindOfListView kindOfView );

	/** Zwraca informacj czy biecym widokiem jest drzewo.\n
	 @return TRUE jeli widokiem jest drzewo, w przeciwnym razie FALSE.
	 */
	bool isTreeView() const { return  (bool)mKindOfView; }

	/** Zwraca informacj czy biecym widokiem jest zwyka lista.\n
	 @return TRUE jeli widokiem jest zwyka lista, w przeciwnym razie FALSE.
	 */
	bool isListView() const { return !(bool)mKindOfView; }

	/** Zwraca informacj o rodzaju biecego widoku.\n
	 @return rodzaj widoku, patrz: @see KindOfListView.
	 */
	KindOfListView kindOfView() const { return mKindOfView; }


	/** Zwraca informacj o tym, czy dla linkw pokazywane dugoci ich cieki,
	 czy rozmiary plikw docelowych.\n
	 @return TRUE - oznacza, e pokazywane s dugoci ich cieki, w przeciwnym
	 razie rozmiaru plikw docelowych.
	 */
	bool showFileLinkSize() const { return mShowFileLinkSize; }

	/** Metoda wcza/wycza pokazywanie rozmiarw dla linkw jako dugo ich
	 cieki.\n
	 @param TRUE - wymusza pokazywanie dugoci ich cieki, w przeciwnym razie
	 rozmiaru pliku docelowego
	 */
	void setShowFileLinkSize( bool showFileLinkSize ) { mShowFileLinkSize = showFileLinkSize; }

	/** Powoduje zastosowanie filtra dla listy plikw.\n
	 @param fTemplate - wzorzec dopasowania plikw,
	 @param dirFilter - typ plikw.\n
	 Po wykonaniu tej metody, na licie pozostan tylko pasujce do wzora i typu,
	 reszta zostanie ukryta.
	 */
	void applyFilter( const QString & fTemplate, Filters::DirFilter dirFilter, bool filterForFiltered=FALSE );

//	UrlInfoList & bufferUrlsList()  { return mBufferUrlsList;         }
//	uint bufferUrlsListCount()      { return mBufferUrlsList.count(); }
//	void setBufferUrlsList( const QValueList<UrlInfoExt> & urlsList ) { mBufferUrlsList = urlsList;        }
//	void appendToBufferUrlsList( const UrlInfoExt & urlInfo )         { mBufferUrlsList.append( urlInfo ); }

	/** Zwraca numer kolumny, po ktrej sortowana jest lista.
	 @return numer sortowanej kolumny.
	 */
	uint sortColumn() { return mSortColumn; }

	/** Zwraca kierunek sortowania.
	 @return TRUE oznacza od najmniejszej do najwikszej wartoci,
	 FALSE - odwrotnie
	 */
	bool ascending()  { return mAscending;  }

	/** Metoda ustawia sortowanie listy wg podanych parametrw.\n
	 @param column - kolumna, po ktrej naley sortowa,
	 @param ascending - kierunek sortowania (TRUE - od najmniejszego do
	 najwikszego, FALSE - odwrotnie).
	 */
	void setSorting( int column, bool ascending );



	/** Zwraca informacj o tym, czy kursor jest nie ukrywany na licie podczas
	 jej przewijania.\n
	 @return TRUE oznacza, e kursor nie jest ukrywany, w przeciwnym razie jest
	 ukrywany.
	 */
	bool focusAlwaysVisible()  { return mNotHiddenFocus; }

	/** Metoda wcza/wycza cige mechanizm nieukrywania kursora na licie
	 podczas jej przewijania.\n
	 @param focusAlwaysVisible - TRUE wymusza nieukrywanie kursora, w przeciwnym
	 razie bdzie on ukrywany.\n
	 Domylnie waciwo ta jest wczona.
	 */
	void setFocusAlwaysVisible( bool focusAlwaysVisible ) { mNotHiddenFocus = focusAlwaysVisible; }


//	void takeItem( const QString & item ) { removeItem(item); };
	/** Funkcja prbuje usunc element o podanej nazwie.\n
	 @param name - nazwa (warto pierwszej kolumny) elementu do usunicia.\n
	 Po udanym usuniciu, kursor jest ustawiany na nastpny element listy.
	 */
	void removeItem( const QString & name );

	/** Funkcja powoduje wstawienie podanego elementu na list.\n
	 @param urlInfo - informacja o elemencie,
	 @param parentItemTree - wskanik elementum ktry bdzie rodzicem dla
	 wstawianego (uywane dla widoku drzewa),
	 @param tryRemoveThisSameItem - rwne TRUE wykonuje prb usunicia elememtu
	 o nazwie podanej w @em urlInfo, przed wstawieniem nowego elementu; warto
	 FALSE umoliwia wstawienie kolejnego elementu o takiej samej nazwie,
	 @param setProgress - rwne TRUE wymusza uaktualnienie paska postpu
	 */
	void insertItem( const UrlInfoExt & urlInfo, QListViewItem *parentItemTree, bool tryRemoveThisSameItem=TRUE, bool setProgress=FALSE );


	/** Funkcja uywana do ustawiania kursora na podanego nazw elementu na
	 licie.\n
	 @param name - nazwa elementu, na ktrym naley umiecic kursor.\n
	 Jeli podana nazwa nie istnieje, wtedy kursor ustawiany jest na pierwszy
	 element listy.
	*/
	void moveCursorToName( const QString & name );


	/** Metoda powoduje ustwienie tekstu dla biecego elementu w podanej
	 kolumnie.\n
	 @param column - numer kolumny, w ktrej naley ustawi tekst,
	 @param str - tekst do ustawienia.
	 */
	void setTextInColumn( int column, const QString & str );

	/** Zwraca cig tekstowy biecego elementu z podanej kolumny.\n
	 @param column - kolumna w ktrej znajduje si tekst do pobrania.
	 */
	QString currentItemText( uint column = 0 ) const { return currentItem()->text( column ); }


	/** Zwraca liczb elementw na licie lub drzewie.\n
	 @param minusOneForList - rwne TRUE wymusza korekt (-1 dla zwracanej liczby
	  elementw). Przydatne jest to dla widoku listy, dziki temu nie jest liczony
	  element '..'.
	 @return liczba elementw listy.
	 */
	uint numOfAllItems( bool minusOneForList=TRUE );


	/** Powoduje zaznaczenie grupy elementw we wskazanym mysz zakresie.\n
	 @param from - wskanik elementu, od ktrego trzeba zacz zaznacza
	  (wcznie),
	 @param to - wskanik elementu, na ktrym naley skoczy zaznaczanie
	  (wcznie),
	 @param clear - rwne TRUE, oznacza odznaczanie, w przeciwnym razie elementy
	  zostan zaznaczone
	 @param invert - rwne TRUE powoduje inwersje zaznaczania na wybranym zakresie
	 */
	void selectRange( QListViewItem * from, QListViewItem * to, bool clear = FALSE, bool invert = FALSE );

	/** Zaznacza lub odznacza element listy.\n
	 @param item - wskanik na element listy,
	 @param select - rwne TRUE, wymusza zaznaczenie, w przyciwnym wypadku element
	 jest odznaczany.\n
	 Uaktualniana jest tu rwnie lista zaznaczonych, a na kocu wysyany sygna
	 @see signalUpdateOfListStatus().
	 */
	void setSelected( QListViewItem * item, bool select );

	/** Metoda zapisuje na list nazwy wraz ze sciekami wszystkie zaznaczone
	 elementy, przy czym jeli adnego nie zaznaczono, wtedy dopisuje tylko nazw
	 biecego.\n
	 @param selectedNamesList - adres listy cigw, na ktrej umieszczone zostan
	 nazwy.
	 */
	void collectAllSelectedNames( QStringList & selectedNamesList );

	/** Zlicza wszystkie zaznaczone pliki (ich liczb oraz cakowit wag), przy
	 czym katalogi nie s waone.\n
	 @param addToTheSelectedItemsList - rwny TRUE, wymusza dodanie zaznaczonego
	 elementu do listy zaznaczonych.\n
	 Na kocu wysyany jest sygna @see signalUpdateOfListStatus().
	 */
	void countAndWeightSelectedFiles( bool addToTheSelectedItemsList=TRUE );

	/** Zwraca liczb wszystkich zaznaczonych elementw.\n
	 @return liczba zaznaczonych elementw.
	 */
	uint numOfSelectedItems() const { return mNumOfSelectedItems; }

	/** Zwraca wag wszystkich zaznaczonych elementw.\n
	 @return waga zaznaczonych.
	 */
	long long weightOfSelected() const { return mWeightOfSelected; }

	/** Zwraca informacj o tym czy w zaznaczanie grupowe obejmuje tylko
	zaznaczanie plikw.\n
	@return TRUE - zaznaczane s tylko pliki, w przeciwnym razie wszystkie
	elementy.
	 */
	bool selectFilesOnly() const { return mSelectFilesOnly; }

	/** Wcza/wycza dla zaznaczania grupowego zaznaczanie tylko plikw.\n
	 @param filesOnly - rwne TRUE, wymusza zaznaczanie tylko plikw, w przeciwnym
	 razie zaznaczane bd wszystkie elementy.
	 */
	void setSelectFilesOnly( bool filesOnly ) { mSelectFilesOnly = filesOnly; }


	/** Zwraca wskanik do obiektu menu podrcznego.\n
	 @return wskanik do obiektu typu @see PopupMenu.
	 */
	PopupMenu *popupMenu() { return mPopupMenu; }

	/** Pokazuje menu podrczne.\n
	 @param onCursorPos - rwne TRUE, wymusza ustawienie menu na wsprzdnych
	 kursora myszy. Warto FALSE powoduje, e pozycj jest: X = prawy brzeg
	 pierwszej kolumny, Y = gorny brzeg biecego elementu listy.
	 */
	void showPopupMenu( bool onCursorPos=TRUE );

	/** Zwraca wskanik do obiektu dialogu szybkiego wyszukiwania elementu na
	 licie.\n
	 @return wskanik do obiektu typu @see ListViewFindItem.
	 */
	ListViewFindItem *findItemDlg() { return mListViewFindItem; } // do wyrzucenia w przyszlosci

	/** Funkcja pokazuje minidialog z okienkiem edycyjnym do wpisywania nazwy
	 szukanego na licie elementu.\n
	 */
	void showFindItemDialog();


	/** Prbuje odnale na licie lub w drzewie element o podanej nazwie.\n
	 @param name - nazwa elementu do odnalezienia,
	 @return wskanik do znalezionego elementu, lub NULL, jeli nie
	 zosta on odnaleziony.
	 Szukanie nastpuje w pierwszej kolumnie listy.
	 */
	QListViewItem * findName( const QString & name );

	/** Zwraca informacj o tym, czy biecy element jest katalogiem.
	 @return TRUE jeli element jest katalogiem, w przeciwnym razie FALSE.
	 Przy czym kady link jest zawsze traktowany jak plik.
	 */
	bool currentItemIsDir() { return ((ListViewItemExt *)currentItem())->isDir(); }

	/** Powoduje, e podany element pojawia si w widocznym obszarze listy.\n
	 @param item - wskanik elementu, ktry naley pokaza.
	 */
	void ensureItemVisible( QListViewItem * item=0 );

	/** Powoduje, e element o podanej nazwie pojawia si w widocznym obszarze
	 listy.\n
	 @param name - nazwa elementu (z pierwszej kolumny), ktry naley pokaza.
	 */
	void ensureNameVisible( const QString & name );


	/** Zwraca ciek z nazw biecego elementu.\n
	 @param absolutePath - rwne TRUE, wymusza zwrcenie cieki absolutnej,
	  w przeciwnym razie zwracana jest wzgldna
	 @param findParent - (uywane dla widoku drzewa); jeli rwny TRUE, wtedy
	 @return cieka biecego elementu.
	 sprawdzane jest czy biecy element jest otwarty, jeli tak to zwracana jest
	 jego cieka. Jeeli biecy element jest zamknity, wtedy zwracana jest
	 cieka jego rodzica. Przy czym dla elementu korznia zwracana jest cieka
	 hosta (zawarta w skadowej @em mHost). Warto FALSE, wtedy zwracana jest
	 cieka biecego elementu.\n
	 Jeeli elementem jest katalog, wtedy do nazwy dodawany jest znak slash ("/").
	 */
	QString pathForCurrentItem( bool absolutePath=TRUE, bool findParent=FALSE );

	/** Zwraca absolutn ciek dla podanego elementu lub jego nazw.\n
	 @param item - wskanik na elementu, dla ktrego naley zwrci ciek; przy
	 czym jeli jest on rwny NULL, wtedy brany jest element biecy,
	 @param addNameWhenDir - rwne TRUE wymusza dodanie nazwy elementu, jeli jest
	 on katalogiem.
	 @return cieka podanego elementu.
	 Dla widoku drzewa, zwracana jest absolutna cieka dla podanego elementu
	 (bez jego nazwy), dla widoku listy jest to nazwa elemetu z pierwszej kolumny.
	 */
	QString absolutePathForItem( QListViewItem * item=0, bool addNameWhenDir=FALSE );


	/** Rodzaj kolumny. */
	enum ColumnName { NAMEcol=0, SIZEcol, TIMEcol, PERMcol, OWNERcol, GROUPcol };

	/** Zwraca szeroko podanej kolumny.\n
	 @param column - numer kolumny, ktrej szeroko naley zwrci,
	 @return szeroko kolumny.
	 */
	int columnSize( uint column ) const { return header()->sectionSize( column ); }

	/** Dodaje kolumn o podanym numerze oraz ustawia jej szeroko.\n
	 @param columnName - numer kolumny,
	 @param columnWidth - szeroko kolumny.\n
	 Jeli podana szeroko jest rwna 0, wtedy kolumna jest usuwana, jeli
	 wiksza ni zero, wtedy podana szeroko jest szerokoci kolumny. Jeeli
	 szeroko jest mniejsza od minimalnej szerokoci dla danej kolumny, wtedy
	 przyjmowana jest warto domylna.
	 */
	void setColumn( ColumnName columnName, uint columnWidth );

	/** Powoduje pokazanie lub ukrycie nagwka z kolumnami.\n
	 @param show - rwne TRUE wymusza pokazanie kolumn, FALSE ukrywa go.
	 */
	void setShowColumns( bool show = TRUE );


	/** Ustawia nazw hosta dla biecego systemu plikw.\n
	 @param hostName - nowa nazwa hosta.
	 */
	void setHostName( const QString & hostName )   { mHost = hostName;  }

	/** Ustawia aktualn ciek w biecym systemie plikw.\n
	 @param url - nowa bieca cieka.
	 */
	void setCurrentURL( const QString & url ) { mCurrentURL = url; }

// 	void forceShowToolTip( const QPoint & pos ) { mToolTip->maybeTip( pos );


	/** Metoda uruchamia procedure ponownego odczytania biecego katalogu.\n
	 Wywoywane s tu kolejno metody @em close() i @em open() .
	 */
	void refresh();

	/** Funkcja powoduje uruchomienie procedury otwarcia pliku.\n
	 @param fileName - nazwa pliku do otwarcia,
	 @param closeIfOpen - rwne TRUE spowoduje zamknicie gazi, jeli ta jest
	 ju otwarta (dziala tylko dla widoku drzewa).\n
	 Inicjowane jest tutaj odpowiednie zachowanie odpowiednio dla widoku drzewa
	 i listy, a po jego zakoczeniu wysyany sygna @em signalOpen() powodujcy
	 wylistowanie lub pokaznie podgldu podanego pliku.
	 */
	void open( const QString & fileName=QString::null, bool closeIfOpen=FALSE );

	/** Metoda uruchamia procedur zamknicia gazi o podanej ciece.\n
	 @param path - cieka dla gazi do zamknicia.\n
	 Jeeli biecym widokiem jest drzewo wtedy ga jest zamykana, jeli
	 widokiem jest lista, wtedy zerowane s tylko skadowe okrelajce liczb
	 i wag zaznaczonych.
	 */
	void close( const QString & path=QString::null );


	/** Metoda powoduje zaincjowanie widoku listy/drzewa, przed listowaniem
	 plikw.\n
	 @param path - cieka biecego katalogu (wykorzystywana do okrelenia czy
	 katalog zawiera urzdzenia),
	 @param numOfAllItems - liczba wszystkich elementw z katalogu,
	 @param owner - nazwa biecego uytkowanika (inicjuje skdow klasy),
	 @param group - nazwa grupy do ktrej naley biecy uytkownik  (inicjuje
	 skadow klasy),
	 @param forceCleaning - rwne TRUE wymusza czyszczenie listy.\n
	 Jeli widokiem bdzie lista, wtedy widok jest czyszczony, zerowana jest lista
	 zaznaczonych oraz skadowe okrelajce ilo zaznaczonych i ich wag, a take
	 wstawiany jest element '..'.
	 */
	void initFilesList( const QString & path, uint numOfAllItems, const QString & owner, const QString & group=QString::null, bool forceCleaning=FALSE );


	/** Rodzaj uaktualnienia listy.
	 */
	enum UpdateListOperation { INSERTitems=1, REMOVEitems=2, DESELECTitems=4, INSERTitemsAsLinks=8 };
//	typedef QPtrList<QListViewItem> SelectedItemsList;
	typedef QValueList<QListViewItem *> SelectedItemsList;

	/** Zwraca wskanik na list zaznaczonych elementw.\n
	 @return wskanik do listy zaznaczonych typu @em SelectedItemsList
	 */
	SelectedItemsList * selectedItemsList() { return mSelectedItemsList; }

	/** Metoda powoduje uaktualnienie listy.\n
	 @param updateListOp - rodzaj uaktualnienia, patrz: @see UpdateListOperation,
	 @param list - wskanik na list elementw, ktrymi naley uaktualni biec,
	 @param newUrlInfo - informacja o pliku (uywana jeli uaktualniany jest jeden
	 element).\n
	 Po wykonaniu operacji, kursor ustawiany jest na ostatni uaktualniony element,
	 po czym wysyany zostaje sygna uaktualniajcy informacj o licie.
	 */
	void updateList( int updateListOp, SelectedItemsList *listOfItems=0, const UrlInfoExt & newUrlInfo=scNullUrlInfoExt );

	/** Metoda zeruje list zaznaczonych elementw.
	 */
	void clearSelectedItemsList() { if ( mSelectedItemsList->count() )  mSelectedItemsList->clear(); }

	/** Metoda usuwa podany element z listy zaznaczonych.\n
	 @param item - wskanik elementu do usunicia.
	 */
	void removeItemFromSIL( QListViewItem * item );


	/** Zwraca numer widocznej kolumny odpowiadajcy podanemu.\n
	 @param column - numer kolumny do sprawdzenie
	 @return numer kolumny
	 */
	int sectionMap( uint column ) const { return (column<mColumnsList.count()) ? mColumnsList[column] : -1; }

private:
	QColor mGridColor, mSecondColorOfBg;
	QColor mTextColorUnderCursor, mTextColorHighlightUnderCursor;
	QColor mFullCursorColor, mFrameInsCursorColor, mFrameOutCursorColor;

	bool mShowGrid, mEnableTwoColorsOfBg;
	bool mFullCursor, mDblFrameCursor;
	bool mNotHiddenFocus;
	bool mShowFileLinkSize;
	bool mShowIcons;
	QString mFontFamily;
	uint mFontSize;
	uint mIconWidth;
	uint mNumOfSelectedItems;
	long long mWeightOfSelected;
	bool mSelectFilesOnly, mInvSelection;
	uint mSortColumn;
	bool mAscending;
	QListViewItem *mCurrentItem;
	QString mHost, mCurrentURL;

	ListViewExtToolTip *mToolTip;
	QListViewItem *mRootsItemOfCurrentPath;
	SelectedItemsList *mSelectedItemsList;

	KindOfFormating mKindOfFormating;
	KindOfListView  mKindOfView;

	bool mDragging;
	QTimer *mScrollTimer;

	ListViewFindItem *mListViewFindItem;
	ListViewEditBox *mListViewEditBox;
	PopupMenu *mPopupMenu;

	bool mCurrentDirIsDevDir;
	uint mInsertedItemsCounter, mNumOfAllItemsPB; // use by settings progressBar

	QWidget *mParent;
	QString mOwner, mGroup; // of listView

	QValueList <int> mColumnsList;

protected:
	/** Przechwytywane s tutaj wszelkie zdarzenia listy.\n
	 @param o - wskanik obiektu, ktrego dotyczy zadarzenie,
	 @param e - wskanik obiektu zdarzenia.
	 W przypadku wystpienia zdarzenia FocusOut i jeli menu podrczne lub dialog
	 szukania elementw jest widoczny - kursor listy jest ustawiany na ni. Dla
	 zdarzenia FocusIn jeli okienko edycyjne jest widoczne lub jeli menu
	 podrczne lub dialog szukania elementw jest widoczny nic nie jest robione,
	 w przeciwnym razie wysyany jest sygna uaktualniajcy widok cieki biec.
	 W przypadku zdarzenia KeyPress, jeli menu podrczne jest widoczne to
	 wcinite klawisze przesyane s do obiektu klasy menu podrcznego. Jeeli
	 dialog szukania elementw jest widoczny, wtedy wcinicie klawiszy F3...F8
	 spowoduje wysanie zdarzenia wcinicia klawisza do obiektu klasy rodzica
	 dla biecej. Przechwytywane s tutaj te niektre klawisze wspierajce
	 obsug drzewa.
	 */
	bool eventFilter( QObject * o, QEvent * e );

	/** Funkcja wywoywana podczas przesuwania kursorem myszy po widoku listy
	 z wcinitym lewym klawiszem.\n
	 @param e - zdarzenie myszy.\n
	 Jest tu uruchamiane automatyczne przewijanie widoku, gdy kursor wyjdzie poza
	 dolny lub grny margines listy.
	*/
	void contentsMouseMoveEvent( QMouseEvent * e );

	/** Obsuga zdarzenia kliknicia lewego lub prawego klawisza myszki.\n
	 @param e - zdarzenie myszy.\n
	 Po klikniciu prawym klawiszem pokazywane jest menu podrczne. Przytrzymanie
	 klawisza SHIFT i kliknicie lewym klawiszem myszy spowoduje zaznaczenie
	 elementu na ktrym stoi kursor,
	*/
	void contentsMousePressEvent( QMouseEvent * e );

	/** Funkcja wywoywana po zwolnieniu przycisku myszy.\n
	 Jest ona niezbdna dla poprawnego dziaania automatycznego przewijania widoku
	 listy, gdy wcinity jest lewy klawisz myszy a kursor jest poza grynym lub
	 dolnym marginesem listy.
	 */
	void contentsMouseReleaseEvent( QMouseEvent * );

	/** Funkcja wywoywana po podwjny klikniciu lewym klawiszem myszy w element
	 listy.\n
	 @param e - zdarzenie myszy.\n
	 Uruchamiana jest tu procedura obsugi dla podwojnego kliknicia (wysyany
	 jest sygna @see returnPressed).
	 */
	void contentsMouseDoubleClickEvent( QMouseEvent * );


	/** Metoda powoduje ustawienia kolorw elementw listy.\n
	 @param r - element do kolorowania,
	 @param c - kolor do ustawienia.\n
	 Uywana jest przez inne metody zmieniajce kolory poszczeglnych elementw.
	*/
	void setColor( QColorGroup::ColorRole, const QColor & );


	/** Otwiera ga o podanej ciece.\n
	 @param absPathName - cieka w drzewie do otworzenia.
	 @param moveViewAtTop - rwne TRUE wymusza przesunicie widoku na sam gr.\n
	 Metoda wysya sygna @em signalOpen() z parametrem cieki, powodujcy
	 wylistowanie zawartoci tego katalogu w systemie plikw.
	 */
	void openBranch( const QString & absPathName=QString::null, bool moveViewAtTop=TRUE );

	/** Usuwa zawarto gazi, o podanej ciece.\n
	 @param absPathName - cieka dla gazi do wyczyszczenia.
	 */
	void closeBranch( const QString & absPathName=QString::null );

public slots:
	/** Zaznacza lub odznacza biecy element oraz dolicza go (ilo, rozmiar) do
	 zaznaczonych/odznaczonych, po czym powoduje uaktualnienie informacji
	 o plikach.
	 */
	void slotSelectCurrentItem();

	/** Zaznacza lub odznacza wszystkie elementy (pomijajc ten o nazwie '..').\n
	 @param clear - rwne TRUE, oznacza e elementy bd odznaczane, w przeciwnym
	 razie zostan zaznaczone.\n
	 Uaktualniana jest tu rwnie informacja o plikach.
	*/
	void slotSelectAllItems( bool clear = FALSE );

	/** Wykonuje inwersje na wszystkich elementach listy ( pomijajc ten o nazwie
	 '..' ).\n Uaktualniana jest tu rwnie informacja o plikach.
	*/
	void slotInvertSelections();

	/** Zaznacza lub odznacza elementy wg podanego wzorca.\n
	 @param select - rwne TRUE, wymusza zanaczanie elementw, w przeciwnym razie
	 zostan odznaczone,
	 @param pattern - wzorzec, wg ktrego nastpi zaznaczenie lub odznaczanie.\n
	 Funkcja zaznaczania uaktualnia rwnie statusow informacj o plikach.
	 */
	void slotSelectByUsePattern( bool clear, const QString & pattern );

	/** Zaznacza lub odznacza pliki z tym samym rozszerzeniem, jak ten, na ktrym
	 aktualnie stoi kursor.\n
	 @param clear - rwne TRUE, oznacza e elementy bd odznaczane, w przeciwnym
	 wypadku zostan zaznaczone.\n
	 Uaktualniana jest tu rwnie informacja o plikach.
	 */
	void slotSelectWithThisSameExtension( bool clear = FALSE );

	/** Pokazuje okno dialogowe suce do pobierania wzorca do zaznaczenia lub
	 odznaczenia elementw.\n
	 @param clear - rwne TRUE, oznacza e elementy bd odznaczane, w przeciwnym
	 razie zostan zaznaczone.\n
	 Po wciniciu 'OK', elementy s zaznaczane lub odznaczane wg podanego wzorca.
	 Mona rwnie poda kilka wzorcw, wtedy naley je oddzieli znakiem "/".
	 Lista wzorcw jest za kadym razem zapisywana do pliku konfiguracyjnego,
	 przy czym ostatnio wybrany lub nowy zawsze znajduje si na pierwszym miejscu.
	*/
	void slotSelectGroupOfItems( bool clear = FALSE );

	/** Slot wywoywany w momencie zmiany rozmiaru podanej kolumny.\n
	 @param section - numer kolumny, ktrej jest zmieniany rozmiar,
	 @param oldSize - poprzedni rozmiar biecej kolumny,
	 @param newSize - nowy rozmiar biecej kolumny.\n
	 Slot jest wykorzystywany przy synchronizacji kolumn, dba te aby okienko
	 edycyjne pokazywane w okrelonej komrce biecego elementu zachowao
	 pozycje.
	 */
	void slotHeaderSizeChange( int section, int oldSize, int newSize );

	/** Metoda powoduje pokazanie na pozycji biecego elementu, w podanej
	 kolumnie okienka edycyjnego.\n
	 @param column - numer kolumny, w ktrej naley pokaza okienko edycyjne.
	 */
	void slotShowEditBox( int column );


	/** Slot uruchamia wstawienie podanego elementu na list.\n
	 @param urlInfo - informacja o elemencie.\n
	 Wewntrz jest wywoywana funkcja wstawiajca element na list.
	 */
	void slotInsertItem( const UrlInfoExt & urlInfo );

private slots:
	/** Funkcja wywoywana po klikniciu w jedn z kolumn.\n
	 @param section - numer kliknitej kolumny.\n
	 Jeli kursor jest niewidoczny, wtedy przesuwany jest widok tak, aby kursor by
	 pokazany. Inicjowane s tu skadowe: kierunku sortowania (@em mAscending) oraz
	 @em mSortColumn (okrela, po ktrej kolumnie jest sortowana lista)
	 */
	void slotHeaderClicked( int section );

	/** Slot wywoywany w momencie przewijania zawartoci panelu.\n
	 @param contentsY - pozycja Y widoku.\n
	 Jeli skadowa @em mNotHiddenFocus ma warto TRUE, wtedy wykonywana jest tutaj
	 korekcja pozycji kursora, jeli ten wyjdzie poza grny lub dolny margines.
	 W efekcie czego kursor jest zawsze widoczny na panelu podczas jego przewijania.
	*/
	void slotChangeVslider( int contentsY );

	/** Slot wywoywany w momencie przewijania suwakiem poziomym listy.\n
	 Nastpuje tutaj tylko odrysowanie biecego elementu, poniewa nie jest to
	 robione automatycznie.
	 */
	void slotChangeHSlider( int );

	/** Slot powoduje automatyczne przesuwanie si zawartoci listy.\n
	 Jeli kursor myszy znajduje poza grnym marginesem, wtedy lista przesuwa si
	 w d, natomiast kiedy jest poza dolnym to lista poda w gr.
	 */
	void slotDoAutoScroll();

signals:
	/** Sygna jest wysyany w momencie, gdy nastpi potwierdzenia (wcinity
	 zostanie Return/Enter) podczas szybkiej zmiany wartoci komrki.\n
	 @param column - numer kolumny, w ktrej nastpuje zmiana,
	 @param oldText - poprzednia warto tekstowa w komrce,
	 @param newText - nowa warto tekstowa dla komrki.\n
	 Uruchamia on procedure podejmujc prbe zmian atrybutu dla biecego pliku.
	 */
	void signalRenameCurrentItem( int column, const QString & oldText, const QString & newText );

	/** Sygna jest wysyany w po uaktualnieniu listy i zmianie katalogu w widoku
	 drzewa.\n
	 @param numOfSelectedItems - liczba aktualnie zaznaczonych elementw,
	 @param numOfAllItems - liczby wszystkich elementw,
	 @param weightOfSelected - waga zaznaczonych elementw.\n
	 Wymusza on uaktualnienie informacji na statusie biecego panela.
	 */
	void signalUpdateOfListStatus( uint numOfSelectedItems, uint numOfAllItems, uint weightOfSelected );

	/** Sygna powoduje uaktualnienie informacji o biecej ciece na jej widoku.\n
	 @param url - nazwa nowej cieki.\n
	 W przypadku widoku drzewa sygna jest wywoywany przy kadej zmienie
	 pozycji kursora na licie. Dla obydwu widokw jest zawsze wywoywany gdy
	 focus zostanie postawiony na licie.
	 */
	void signalPathChanged( const QString & url );

	/** Sygna jest wysyany w momencie wstawiania elementw na list.\n
	 @param doneInPercent - warto procentowa wstawionych ju elementw,
	 w stosunku do wszystkich z biecego katalogu.\n
	 Powoduje on uaktualnianie paska postpu dla listowania biecego katalogu.
	 */
	void signalSetProgress( int doneInPercent );


	/** Sygna jest wysyany podczas prby otworzenia biecego pliku lub katalogu.\n
	 @param fileName - nazwa pliku lub katalogu, ktry naley otworzy.\n
	 Powoduje on uruchomienie procedury zmiany katalogu na podany lub otwarcia
	 podgldu pliku.
	 */
	void signalOpen( const QString & fileName );

	/** Sygna jest wysyany tylko na widoku drzewa w przypadku, gdy bieacy
	 katalog lub archiwum jest zamykane.
	 */
	void signalClose(); // current list

};



inline void ListViewExt::setSecondColorOfBg( const QColor & c )
{
	mSecondColorOfBg = c;  triggerUpdate();
}

inline void ListViewExt::setTextColorUnderCursor( const QColor & c )
{
	mTextColorUnderCursor = c;  triggerUpdate();
}

inline void ListViewExt::setHighlightTextColorUnderCursor( const QColor & c )
{
	mTextColorHighlightUnderCursor = c;  triggerUpdate();
}

inline const QColor & ListViewExt::secondColorOfBg() const     { return mSecondColorOfBg;     }
inline const QColor & ListViewExt::gridColor() const           { return mGridColor;           }
inline const QColor & ListViewExt::cursorColor() const         { return mFullCursorColor;      }
inline const QColor & ListViewExt::frameInsCursorColor() const { return mFrameInsCursorColor; }
inline const QColor & ListViewExt::frameOutCursorColor() const { return mFrameOutCursorColor; }

inline bool ListViewExt::grid() const                 { return mShowGrid;            }
inline bool ListViewExt::enabledTwoColorsOfBg() const { return mEnableTwoColorsOfBg; }
inline bool ListViewExt::isFullCursor() const         { return mFullCursor;          }

inline void ListViewExt::setShowGrid( bool show )               { mShowGrid = show;	     triggerUpdate();       }
inline void ListViewExt::setEnableTwoColorsOfBg( bool enable )  { mEnableTwoColorsOfBg = enable; triggerUpdate(); }
inline void ListViewExt::setGridColor( const QColor & c )       { mGridColor = c;        triggerUpdate();       }
inline void ListViewExt::setFullCursorColor( const QColor & c ) { mFullCursorColor = c;   triggerUpdate();       }

inline const QColor & ListViewExt::highlightBgColor() const      { return colorGroup().highlight();       }
inline const QColor & ListViewExt::highlightedTextColor() const  { return colorGroup().highlightedText(); }
inline const QColor & ListViewExt::textColor() const             { return colorGroup().text();            }
inline const QColor & ListViewExt::textColorUnderCursor() const  { return mTextColorUnderCursor;          }
inline const QColor & ListViewExt::highlightedTextColorUnderCursor() const  { return mTextColorHighlightUnderCursor; }

inline void ListViewExt::setFullCursor( bool fullCursor )
{
	mFullCursor = fullCursor;  mDblFrameCursor = false;
}

inline void ListViewExt::setDblFrameCursor( bool dblFrameCursor )
{
	mDblFrameCursor = dblFrameCursor;  mFullCursor = false;
}

inline void ListViewExt::setCursorColor( const QColor & c )
{
	setFullCursorColor( c );  setDblFrameCursorColors( c, c );
}

inline void ListViewExt::setDblFrameCursorColors( const QColor & colInside, const QColor & colOutside )
{
	mFrameInsCursorColor = colInside;  mFrameOutCursorColor = colOutside;
}

#endif
