/***************************************************************************
                          qtcmd.h  -  description
                             -------------------
    begin                : wed oct 23 2002
    copyright            : (C) 2002 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef QTCMD_H
#define QTCMD_H

#include <qmainwindow.h>
#include <qtoolbutton.h>
#include <qpushbutton.h>
#include <qpopupmenu.h>
#include <qsplitter.h>
#include <qtoolbar.h>
#include <qlibrary.h>
#include <qaccel.h>

#include "enums.h"
#include "panel.h"
#include "filters.h"
#include "keyshortcuts.h"

#include "plugins/fp/settings/listviewsettings.h"
#include "plugins/fp/settings/otherappsettings.h"
#include "plugins/fp/settings/filesystemsettings.h"

/**
 * Author:
 * Piotr Mierzwiski
 **/
/**
 @short Klasa gwna projektu - pokazuje okno gwne aplikacji.
 Inicjowane s tutaj wszystkie pokazywane w oknie gwnym obiekty (pasek menu,
 narzdziowy oraz panele). Znajduj si tutaj te sloty pomocnicze przy
 komunikacji obydwu paneli.
 */
class QtCmd: public QMainWindow
{
	Q_OBJECT
public:
	/** Konstruktor okna gwnego aplikacji.\n
	 @param leftURL - cieka lub nazwa pliku dla lewego panelu,
	 @param rightURL - cieka lub nazwa pliku dla prawego panelu.\n
	 Inicjuje gowne ustawienia aplikacji (rozmiar, rodzj panelw, informacje o
	 pokazaniu paska narzdziowego), menu gwne, pasek narzdziwy oraz zarzdce
	 widoku paneli.
	 */
	QtCmd( const QString & leftURL=QString::null, const QString & rightURL=QString::null );

	/** Destruktor klasy.\n
	 Usuwane s tutaj obiekty paneli i inne uywane przez klase. Nastpuje take
	 zapisanie ustawie (szerokoci, wysokoci, statusu pokazywania paska menu
	 oraz narzdziowego) do pliku konfiguracyjnego. Poza tym wyadowywany jest
	 plugin ustawie aplikacji.
	 */
	~QtCmd();

private:
	uint mWidth, mHeight;
	int  mWidthOfLeftPanel, mWidthOfRightPanel;

	QSplitter *mSplitter;

	QToolBar *mToolBar, *mButtonBar;
	QPopupMenu  *mViewMenu, *mKindOfVFPmenu, *mProportionsOfPanelsMenu, *mFavoritesMenu;
	QToolButton *mChangeKindOfViewToolBtn, *mFTPtoolBtn, *mRereadBtn, *mProportionsBtn, *mFreeSpaceBtn, *mDevicesBtn, *mFavoritesBtn, *mQuickGetOutBtn, *mSettingsBtn;

	Panel *mLeftPanel, *mRightPanel;
	FilesAssociation *mFilesAssociation;
	Filters *mFilters;

	QString   mPluginsPath;
	QLibrary *mSettingsLib;
	QWidget  *mSettingsWidget;
	KeyShortcuts *mKeyShortcuts;
	QAction *mQuitAppsA, *mMenuBarA, *mToolBarA, *mBtnBarA, *mConfigureA, *mFtpMmngA, *mFreeSpaceA, *mStorageA, *mRereadA, *mFavoritesA, *mFindFilesA, *mQuickGetOutA;

	enum CurrentPanel { LEFTpanel=0, RIGHTpanel, NOpanel };

	enum PanelProportion { PP_5050=21, PP_3070, PP_7030, PP_USER };
	bool mMenuBarVisible, mToolBarVisible;
	bool mButtonBarVisible, mFlatButtonBar;
	bool mShowQuitWarn;

	enum ButtonBarKeys {
	 HelpBTN=0, PanelViewBTN, FileViewBTN, FileEditBTN, CopyBTN, MoveBTN, MkDirBTN, DeleteBTN, QuitBTN
	};
	QPushButton *mButtonsTab[10];


	/** Inicjowanie okna gwnego aplikacji.\n
	 Odczytuje z pliku konfiguracyjnego rozmiar okna gwnego oraz przypisuje te
	 paremetry aplikacji. Jeli wymagane parametry nie zostay znalezione w pliku
	 konfiguracyjnym, zapisywane s w nim domylne, a aplikacja nimi inicjowana.
	 Odczytywane s rwnie szerokoci obu paneli.
	 */
	void initMainSettings();

	/** Funkcja inicjuje menu gwne aplikacji oraz pasek narzdziowy.
	 */
	void initMenuAndToolBar();

	/** Inicjowane s tutaj pod-menu dla opcji: "Rodzaj widoku dla biecego
	 panela" oraz dla "Proporcje paneli".
	 */
	void initSubMenus();

	/** Metoda tworzy i inicjuje pasek zawierajcy guziki powizane z
	 podstawowymi akcjami na plikach.\n
	 Pasek jest umieszczany domylnie u dou okna.
	 */
	void initButtonsBar();

	/** Metoda powoduje utworzenie obiektu skrtw klawiszowych oraz
	 zainicjowanie jego listy domylnymi wartociami.
	 */
	void initKeyShortcuts();

	/** Inicjuje dwa panele.\n
	 Tworzone s tutaj obiekty obydwu paneli oraz obiekt splittera, ktry jest
	 rodzicem paneli.\n
	 @param leftURL - cieka lub nazwa pliku dla lewego panelu,
	 @param rightURL - cieka lub nazwa pliku dla prawego panelu.
	 */
	void createPanels( const QString & leftURL, const QString & rightURL );

	/** Funkcja zwraca numer biecego panelu.\n
	 @return numer biecego panelu (dla lewego jest to 'LEFTpanel', a prawy
	 jest rwny 'RIGHTpanel').\n
	 Uywana jest przez sloty uywane do komunikacji pomidzy panelami.
	 */
	uint currentPanel();

protected:
	/** Obsuga zdarzenia zmiany rozmiaru dla gwnego okna aplikacji.
	 */
	void resizeEvent( QResizeEvent * );

private slots:
	/** Pokazuje okienko informacyjne z nazw, przeznaczeniem programu, autorach,
	 biecej wersji oraz prawach autorskich.
	 */
	void about();

	/** Pokazuje informacj o wersji biblioteki Qt zlinkowanej z programem.
	 */
	void aboutQt();

	/** Slot wykonuje zamknicie aplikacji.
	 */
	void slotQuit();

	/** Slot powoduje pokazanie okna pomocy.
	 */
	void slotShowHelp();

	/** Slot uruchamia zmian widoku.\n
	 Uywany przez pasek guzikw.
	 */
	void slotChangeView();

	/** Slot uruchamia podgld biecego pliku.\n
	 Uywany przez pasek guzikw.
	 */
	void slotFileView();

	/** Slot uruchamia edycj biecego pliku.\n
	 Uywany przez pasek guzikw.
	 */
	void slotFileEdit();

	/** Slot uruchamia kopiowanie plikw.\n
	 Uywany przez pasek guzikw.
	 */
	void slotCopy();

	/** Slot uruchamia przenoszenie plikw.\n
	 Uywany przez pasek guzikw.
	 */
	void slotMove();

	/** Slot uruchamia tworzenie folderu w biecym katalogu.\n
	 Uywany przez pasek guzikw.
	 */
	void slotMakeDir();

	/** Slot uruchamia usuwanie plikw.\n
	 Uywany przez pasek guzikw.
	 */
	void slotDelete();


	/** Slot uruchamia utworzenie linku() do wybranych plikw.
	 Uywany przez pasek guzikw.
	 */
	void slotMakeLink();


	/** Slot wywoywany w momencia podjcia prby zmiany aktualnego widoku panela
	 na list.\n
	 Wysyany jest tutaj tylko sygna uruchamiajcy powysz zmian.
	 */
	void slotChangeToListView();

	/** Slot wywoywany w momencia podjcia prby zmiany aktualnego widoku panela
	 na drzewo.\n
	 Wysyany jest tutaj tylko sygna uruchamiajcy powysz zmian.
	 */
	void slotChangeToTreeView();

	/** Slot powoduje ukrycie lub pokazanie paska menu.\n
	 Stan pokazywania zaley od aktualnej wartoci skadowej @em mMenuBarVisible ,
	 ktra jest te tutaj modyfikowana, tak e nastpne wywoanie tego slota
	 bdzie miao skutek przeciwny do poprzedniego.
	 */
	void slotShowHideMenuBar();

	/** Slot powoduje ukrycie lub pokazanie paska narzdziowego.\n
	 Stan pokazywania zaley od aktualnej wartoci skadowej @em mToolBarVisible ,
	 ktra jest te tutaj modyfikowana, tak e nastpne wywoanie tego slota
	 bdzie miao skutek przeciwny do poprzedniego.
	 */
	void slotShowHideToolBar();

	/** Slot powoduje ukrycie lub pokazanie dolnego paska z guzikami.\n
	 Stan pokazywania zaley od aktualnej wartoci skadowej @em mButtonBarVisible ,
	 ktra jest te tutaj modyfikowana, tak e nastpne wywoanie tego slota
	 bdzie miao skutek przeciwny do poprzedniego.
	 */
	void slotShowHideButtonBar();

	/** Slot powoduje zmian rozmiarw paneli na t o podanym Id.\n
	 @param proportionId - id dla nowego rozmiaru. Dostpne s nastpujce
	 wartoci parametru :
	  @li PP_5050 - 50/50 (procentowo),
	  @li PP_3070 - 30/70 (procentowo),
	  @li PP_7030 - 70/30 (procentowo),
	  @li PP_USER - Zdefiniowane przez uytkownika, (ustawione przy starcie
	  aplikacji).\n
	  Slot uywany przez menu guzika sucego do zmiany rozmiarw paneli.
	 */
	void slotChangeProportionOfPanels( int proportionId );


	/** Slot wykonuje prb zaadowania (jeli jest zaadowany to tylko pokazuje)
	 pluginu dialogu ustawie aplikacji, po czym go pokazuje.\n Jeeli nie udao
	 si znale pliku plugina, wtedy pokazywany jest stosowny komunikat.
	 */
	void slotShowSettings();

	/** Slot pokazuje dialog sucy do konfigucji paska narzdziowego (dodawania,
	 przestawiania lub usuwania ikon na nim).\n
	 UWAGA: Metoda nie ma jeszcze definicji !
	 */
	void slotShowConfigureToolBarDlg();

public slots:
	/** Slot powoduje otwarcie podanego pliku lub katalogu w panelu obok.\n
	 @param fileName - nazwa pliku lub katalogu do otwarcia.\n
	 Przy czym to czy bdzie otwarty plik czy katalog zaley od wystpienia znaku
	 '/' na kocu podanej nazwy. Jeli bdzie tam znak '/', wtedy nastpi prba
	 otwarcia katalogu w przeciwnym razie pliku.
	 */
	void slotOpenIntoPanelBeside( const QString & fileName );

	/** Slot wywoywany w momencie zmiany rozmiaru kolumn.\n
	 @param section - kolumna, ktra ma zmieniany rozmiar,
	 @param oldSize - poprzedni (stary) rozmiar kolumny podanej w @em section ,
	 @param newSize - nowy rozmiar kolumny podanej w @em section .\n
	 Uywany jest do synchronizowania zmian szerokoci kolumn w obu panelach.
	 */
	void slotHeaderSizeChange( int section, int oldSize, int newSize );

	/** Slot powoduje ustawienie podanej cieki biecego panela w drugim.\n
	 @param url - cieka do ustawienia.
	 */
	void slotSetPanelBesideURL( const QString & url );

	/** Slot uywany do uaktualniania na licie w drugim panelu informacji o pliku.\n
	 @param fileName - nazwa pliku do uaktualniania.\n
	 Jest wykorzystywany przy edycji dla podgladu pliku osadzonego w oknie gownym
	 aplikacji. \n \n UWAGA: brak definicji !
	 */
	void slotUpdateItemOnLV( const QString & fileName );

	/** Slot zamyka widok w biecym panelu i tworzy tam nowy typu FILESpanel.\n
	 Jest on uywany do usuwania widoku podgldu pliku w biecym panelu.
	 */
	void slotCloseView();

	/** Slot pobiera w podanym parametrze rodzaj systemu plikw jaki si znajduje
	 w drugim panelu.\n
	 @param kindOfFS - zmienna w jakiej zostanie zapisany rodzaj systemu plikw.
	 */
	void slotGetKindOfFSInPanelBeside( KindOfFilesSystem & kindOfFS );

	/** Slot powoduje wywoanie metody uruchamiajcej kopiowanie w panelu obok.\n
	 @param filesList - lista plikw do skopiowania lub przeniesienia,
	 @param move - jeli zawiera TRUE, wtedy biec operacj bedzie przenoszenie,
	 w przeciwnym razie jest to kopiowanie.
	 */
	void slotCopyFiles( const QStringList & filesList, bool move );

	/** Slot wywouje funkcje powodujc uaktualnienie listy w panelu obok.\n
	 @param updateListOp - rodzaj operacji uaktualniania. patrz:
	  @see ListViewExt::UpdateListOperation,
	 @param itemsList - wskanik do wskanika na list z elementami do
	 uaktualnienia,
	 @param newUrlInfo - informacje o pliku (uywane jeli jest jeden pliku
	 uaktualniany).
	 */
	void slotUpdateSecondPanel( int updateListOp, QValueList<QListViewItem *> *itemsList, const UrlInfoExt & newUrlInfo );

	/** Slot ustawia w podanym parametrze adres wskanika do listy zaznaczonych
	 elementw w drugim panelu.\n
	 @param selectedItemsList - adres wskanika do listy zaznaczonych
	 */
	void slotGetSILfromPanelBeside( QValueList<QListViewItem *> *& selectedItemsList );

	/** Ustawia skrty klawiszowe dla akcji w biecym obiektcie (oknie gwnym).
	 */
	void slotApplyKeyShortcuts( KeyShortcuts *keyShortcuts );

	/** Slot ustawia filtry dla listy plikw w obydwu panelach.
	 */
	void slotApplyFilters( Filters * filters );

signals:
	/** Sygna uruchamia zmian biecego widoku w biecym panelu na podany.\n
	 @param kindOfView - rodzaj nowego widoku. Dla widoku listy dozwolne s:
	 LISTpview oraz TREEpview.\n
	 Sygna jest uywany przez sloty @em slotChangeToListView oraz
	 @em slotChangeToTreeView .
	 */
	void signalSetNewKindOfView( uint kindOfView );

	/** Sygna uruchamia tzw. szybkie wyjcie z biecego systemu plikw.\n
	 Przy czym "wyskoczy" mona tylko z nie lokalnego systemu plikw.
	 */
	void signalQuickGetOutFromFS();

	/** Sygna powoduje pokazanie dialogu zarzdzania poczeniami FTP.
	 */
	void signalShowFtpManagerDlg();

	/** Sygna uruchamia ponowne odczytanie biecego katalogu w aktywnym panelu.
	 */
	void signalRereadCurrentDir();

	/** Sygna powoduje pokazanie w lewym grnym rogu biecego panelu - menu
	 z 'ulubionymi katalogami'.
	 */
	void signalShowFavorites();

	/** Sygna powoduje pokazanie w lewym grnym rogu biecego panelu - menu
	 z prost statystyk opisujc cakowite i dostpne miejsce w biecym
	 katalogu.
	 */
	void signalShowFreeSpace();

	/** Sygna powoduje pokazanie w lewym grnym rogu biecego panelu - menu
	 z list dostpnych urzdze masowych (informacja odczywytana jest z pliku
	 '/etc/fstab').
	 */
	void signalDevicesList();

	/** Sygna powoduje pokazanie dialog sucego do wyszukiwania plikw.
	 */
	void signalFindFile();


	/** Sygna uruchamia zastosowanie ustawie dla widoku listy.
	 @param lvs - struktura zawierajca ustawienia
	 */
	void signalApplyListViewSettings( const ListViewSettings & lvs );

	/** Sygna uruchamia zastosowanie ustawie dla pozostaych ustawie.
	 @param oas - struktura zawierajca ustawienia
	 */
	void signalApplyOtherAppSettings( const OtherAppSettings & oas );

	/** Sygna uruchamia zastosowanie ustawie dla systemu plikw.
	 @param fss - struktura zawierajca ustawienia
	 */
	void signalApplyFileSystemSettings( const FileSystemSettings & fss );

	/** Sygna uywany do inicjowania skrtw klawiszowych w dialogu
	 konfiguracyjnym.\n
	 @param keyShortcuts - wskanik na obiekt skrtw klawiszowych.
	 */
	void signalSetKeyShortcuts( KeyShortcuts * keyShortcuts );

	/** Sygna uywany do inicjowania skojae dla plikw w dialogu
	 konfiguracyjnym.\n
	 @param fa - wskanik na obiekt skojae dla plikw.
	 */
	void signalSetFilesAssociation( FilesAssociation *fa );

};

#endif
