/***************************************************************************
                          virtualfs.h  -  description
                             -------------------
    begin                : mon oct 28 2002
    copyright            : (C) 2002 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef _VIRTUALFS_H_
#define _VIRTUALFS_H_

//#include "filesassociation.h" // included in "findfiledialog.h"
#include "propertiesdialog.h"
#include "progressdialog.h"
#include "findfiledialog.h"
#include "findcriterion.h"
#include "listviewext.h"
#include "urlinfoext.h"
#include "enums.h"

#include <qurloperator.h>
#include <qvaluelist.h>
#include <qprocess.h>
#include <qwidget.h>

static bool sCloseProgressDlgAfterFinished;

/**
 * Author:
 * Piotr Mierzwiski
 **/
/**
 @short Klasa bazowa systemu plikw, po ktrej dziedzicz wszystkie FileSystemy.
 Spenia ona wiele zada porednich (poprzez metody wirtualne) przy uruchamianiu
 operacji plikowych, jest najwysz warstw pomidzy UI a systemem plikw.
 Wykonywane s tutaj wstpne czynnoci przed operacjami plikowymi, np. zbieranie
 nazw plikw na list, tworzenie, pokazywanie i chowanie dialogw oraz czynnoci
 inicjalizacyjne dla widoku listy (konieczne do wykonanie niektrych operacji).
 Wykonywane jest tutaj te wstpne sprawdzanie bdw. Znajduje si tutaj rwnie
 obsuga dekodowania po tzw. szybkiej zmianie niektrych atrybutw pliku, np.
 praw dostpu, czy czasu modyfikacji, podawanej wprost przez uytkownika. Obiekt
 tej klasy przechowuje ciek do aktualnie otwartego katalogu w danym systemie
 plikw. Zawiera rwnie metody wykrywajce rodzaj systemu plikw na podstawie
 podanej cieki.
*/
class VirtualFS : public QWidget
{
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param inputURL - nazwa katalogu ktry bdzie otwarty w biecym systemie
	 plikw. Tutaj jest wykorzystywany do wykrywania biecego systemu plikw,
	 @param listViewExt - wskanik na widok listy plikw.\n
	 Inicjowane s tu skadowe klasy i wczytywane ustawienia systemu plikw z pliku
	 konfiguracyjnego.
	 */
	VirtualFS( const QString & inputURL, ListViewExt * listViewExt );

	/** Destruktor.\n
	 Usuwane s tutaj nastpujce obiekty: dialog postpu operacji, dialog
	 wyszukiwania plikw i dialog waciwoci pliku.
	 */
	virtual ~VirtualFS();

	/** Zwraca nazw (ze ciek absolut) aktualnie przetwarzanego pliku.
	 @return nazwa aktualnie przetwarzanego pliku.
	 */
	virtual QString nameOfProcessedFile() const { return QString::null; }

	/** Zwraca kod bdu dla ostatniej operacji plikowej zgodny z @em Error .
	 @return kod bdu.
	 */
	virtual Error errorCode()  const { return NoError; }

	/** Zwraca nazw gwnego katalogu.\n
	 @return nazwa gwnego katalogu.
	 */
	virtual QString host()     const { return QString::null; }

	/** Zwraca biec ciek absolutn.\n
	 @return bieca cieka absolutna.
	 */
	virtual QString path()     const { return QString::null; }

	/** Zwraca numer portu na ktrym obsugiwany jest systemu plikw.\n
	 @return numer portu.\n
	 Uywane tylko w systemie FtpFS.
	 */
	virtual    uint port()     const { return 0; }


	/** Metoda uruchamia procedur tworzenia nowego katalogu lub struktury
	 katalogw o podanej nazwie.
	 */
	virtual void mkdir( const QString & )  {}

	/** Wywoanie funkcji pobierajcej informacje o systemie plikw dla podanego
	 katalogu.
	 */
	virtual bool getStatFS( const QString & , long long & , long long & ) { return FALSE; }


	/** Metoda uruchamia waenie podanych na licie plikw.
	 */
	virtual void sizeFiles( const QStringList &, bool ) {}

	/** Wywoanie funkcji pobierajcej wyniki operacji waenia, tj. waga caoci,
	 ilo plikw i katalogw.
	 */
	virtual void getWeighingStat( long long & , uint & , uint & ) {}

	/** Zwraca informacj o tym czy biecy katalog ma prawa zapisu dla aktualnie
	 zalogowanego uytkownika.
	 */
	virtual DirWritable currentDirWritable() { return UNKNOWNdw; }


	/** Pobiera informacj o pliku wczeniej zainicjowan w skadowej klasy
	 zawierajcej informacje o pliku.
	 */
	virtual void getNewUrlInfo( UrlInfoExt & ) {}

	/** Uruchamia ponowne odczytanie biecego katalogu.\n
	 Wywoywana jest tu metoda z klasy widoku listy plikw.
	 */
	void rereadCurrentDir();

	/** Metoda uruchamia procedur tworzenia pustego pliku lub katalogu.\n
	 @param createDir - warto TRUE wymusza utworzenie katalogu, w przeciwnym
	 razie tworzony jest pusty plik (warto domylna to FALSE).\n
	 Pokazywany jest tutaj te dialog wstpny dla biecej operacji.
	 */
	void createNewEmptyFile( bool createDir=FALSE );

	/** Inicjuje otwarcie zaznaczonych plikw.\n
	 @param fileName - nazwa pliku do otwarcia,
	 @param editMode - warto TRUE wymusza otwarcie biecego pliku w trybie
	  edycji.\n
	 Nazwy plikw do otwarcia zbierane s na list, jeli dany plik nie mona,
	 odczyta wtedy pokazywany jest komunikat bdu. Zebrane pliki otwierane s w
	 wewntrznej lub zewntrznej przegldarce w zalenoci od ustawienia w bazie
	 skojae danego pliku.
	 */
	void openFile( const QString & fileName, bool editMode=FALSE );

	/** Metoda uruchamia procedur kopiowania lub przenoszenia plikw.\n
	 @param targetPath - cieka docelowa kopiowania lub przenoszenia
	 @param filesList - lista nazw plikw do skopiowania lub przenoszenia
	 @param removeSrc - rwne TRUE, wymusza usunicie plikw rdowych po ich
	 skopiowaniu (operacja przenoszenia), w przeciwnym razie pliki s tylko
	 kopiowane.\n
	 Pokazywany jest rwnie wstpny dialog dla tej operacji, a jeli kopiowany
	 lub przenoszony jest jeden plik to nastpuje sprawdzenie jest czy ma tak
	 sam nazw jak docelowy, jeli tak to pokazywany jest komunikt ostrzegawczy
	 i nastpuje powrt do dialogu wstpnego operacji.
	 */
	bool copyFilesTo( const QString & targetPath, const QStringList & filesList, bool remove=FALSE );

	/** Metoda uruchamia procedur usuwania plikw z aktualnego systemu plikw.\n
	 Zbierane s tutaj na list nazwy plikw do usunicia, a przed uruchomieniem
	 operacji pokazywany jest dialog z pytaniem o potwierdzenia usuwania.
	 */
	void removeFiles();

	/** Metoda uruchamia procedur tworzenia lub edycji linku.\n
	 @param targetPath - katalog docelowy dla linku
	 @param editOnly - rwne TRUE, oznacza e bdzie to operacja edycji linku,
	  w przeciwnym wypadku wykonywane jest tworzenie.\n
	 Zbierane s na list nazwy plikw, dla ktrych naley wykona linki.
	 Pokazywany jest rwnie wstpny dialog dla tej operacji.
	 */
	void makeLink( const QString & targetPath, bool editOnly );

	/** Metoda uruchamia procedur zmiany atrybutu (w podanej kolumnie) dla
	 biecego pliku.\n
	 @param column - kolumna w ktrej naley zmieni atrybut pliku,
	 @param newAttribStr - cig opisujcy now warto atrybutu.
	 */
	void changeAttribut( ListViewExt::ColumnName column, const QString & newAttribTxt );

	/** Zwraca docelow nazw pliku, ktremu zmieniono nazw,
	 */
	QString newAttributName() { return mTargetURL; }

	/** Funkcja powoduje pokazanie dialogu waciwoci dla biecego pliku.
	 */
	void showPropertiesDialog();

	/** Funkcja powoduje pokazanie dialogu wyszukiwania pliku.
	 */
	void showFindFileDialog();


	/** Funkcja zajmuje si zainicjowaniem i pokazaniem dialogu postpu dla podanej
	 operacji.\n
	 @param operation - aktualnie wykonywana operacja
	 @param move - jeli rwne TRUE, a operacj jest kopiowanie, wtedy ustawia
	 @param targetPath - cieka docelowa dla biecej operacji (niezbdne dla
	  kopiowania lub przenoszenia).
	 */
	void showProgressDialog( Operation operation, bool move=FALSE, const QString & targetPath=QString::null );

	/** Funkcja powoduje usunicie dialogu postpu.\n
	 @param checkWhetherCanToClose - rwne TRUE, wymusza sprawdzenie informacji o
	  tym czy uytkownik wybra zamknicie na dialogu postpu.
	 */
	void removeProgressDlg( bool checkWhetherCanToClose=FALSE );


	/** Na podstawie podanej nazwy wykrywa rodzaj docelowego systemu plikw.\n
	 @param inputURL - nazwa pliku lub katalogu.\n
	 Jeli jest on rny od biecego, wtedy zwraca URL, w przeciwnym razie jest
	 to warto pusta.\nDodatkowo, jeeli biecy i docelowy system plikw jest
	 taki sam, wtedy porwnywana jest cieka. Jeli jest rna od biecej,
	 wtedy nastpuje prba otwarcia katalogu (lub archiwum).
	 */
	bool needToChangeFS( QString & inputURL );

	/** Zwraca nazw (pliku lub katalogu) z absolutn ciek dostpu biecego
	 FS-u dla podanego pliku lub katalogu.\n
	 @param fileName - nazwa pliku lub katalogu,
	 @return nazwa z absolutn ciek dla biecego systemu plikw.
	*/
	QString absoluteFileName( const QString & fileName ) const;

	/** Zwraca cig bdcy nazw katalogu root aktualnego systemu plikw.\n
	 @return katalog korzenia dla biecego systemu plikw.\n
	 Przykadowo dla lokalnego systemu plikw zwracane jest: "/", natomiast dla
	 FTP - "ftp://", itp. Rodzaj FileSystemu zawarty jest w skadowej
	 @em mCurrentFilesSystem .
	 */
	QString rootName() const;

	/** Funkcja inicjuje skadow @em mCurrentURL podan wartoci parametru.\n
	 @param fullPathName - nazwa, ktr naley zainicjowa biec ciek.
	 */
	void setCurrentURL( const QString & fullPathName );

	/** Zwraca nazw poprzedniego katalogu widocznego na licie.
	 */
	QString previousDirName() const { return mPreviousDirName; }

	/** Zwraca absolutn ciek do biecego katalogu.
	 */
	QString currentURL() const { return mCurrentURL; }

	/** Zwraca absolutn nazw docelowego pliku w operacji zmiany nazwy.
	 */
	QString targetURL() const { return mTargetURL; }

	/** Zwraca absolutn biec ciek dla aktualnego systemu plikw.\n
	 @return absolutna cieka.\n
	 Przy czym dla archiwum zwracana jest tu tylko jego nazwa.
	 */
	QString absHostURL() const;


	/** Metoda zwraca informacj, czy lista plikw zawiera focus.\n
	 @return TRUE jeli lista zawiera focus, FALSE jeli go nie zawiera.
	 */
	bool hasFocus() const { return mListViewExt->hasFocus(); }

	/** Funkcja zwraca rodzaj systemu plikw, na podstawie podanej nazwy.\n
	 @param fullFileName - cieka do katalogu,
	 @return rodzaj systemu plikw, patrz: @see KindOfFilesSystem.
	 */
	static KindOfFilesSystem kindOfFilesSystem( const QString & fullFileName );

	/** Metoda zwraca rodzaj biecego systemu plikw.\n
	 @return rodzaj systemu plikw, patrz: @see KindOfFilesSystem.
	 */
	KindOfFilesSystem kindOfCurrentFS() const { return mCurrentFilesSystem; }

	/** Metoda zwraca rodzaj widoku listy w panelu.
	 @return rodzaj widoku listy, patrz: @see KindOfListView.
	 */
	KindOfListView kindOfViewFilesPanel() const { return mListViewExt->kindOfView(); }


	/** Funkcja powoduje przygotowanie obiektu klasy ListViewExt w celu pokazania
	 na nim nowej listy elementw.
	 */
	void initializeShowsList();


	/** Metoda wykonuje ewentualn korekcj podanej cieki.\n
	 @param path - cieka do poprawienia,
	 @param alwaysAddSlash - rwne TRUE, wymusza dodanie znaku slash ('/') na
	 kocu cieki.\n
	 Zamieniane s tutaj na waciw ciek cigi znakw '../' i './' oraz znaki:
	 '~' (katalog domowy uytkownika), '.' (katalog biecy). Cigi slashy ('//')
	 zamieniane s na pojedyncze znaki '/'. Jeli podana cieka nie jest
	 absolutna, wtedy jest uzupeniana biec.
	 */
	void parsePath( QString & path, bool alwaysAddSlash=TRUE );


	/** Funkcja zwraca nazw podanego kodu operacji na plikach.\n
	 @param operation - kod operacji.
	 @return nazwa operacji.
	 */
	static QString cmdString( Operation operation ); // TYLKO DLA TESTOW


	/** Metoda powoduje ustawienie wagi pliku(w) na dialogu waciwoci.\n
	 Waga jest pobierana z biecego systemu plikw za pomoc wirtualnej funkcji.
	 */
	bool setWeighInPropetiesDlg();

	/** Metoda ustawia status zakoczenia operacji.\n
	 @param operationHasFinished - TRUE, oznacza e operacja zostaa zakoczona,
	 FALSE mwi e operacja nie jest zakoczona.\n
	 Metoda jest wywoywana w slocie 'slotDone' klasy aktualnego systemu plikw.
	 */
	void setOperationHasFinished( bool operationHasFinished ) { mOperationHasFinished = operationHasFinished; }

	/** Metoda wykonuje inicjalizacj na dialogu waciwoci pliku podan
	 informacj o pliku.\n
	 @param urlInfo - adres obiektu z informacj o pliku,
	 @param loggedUserName - nazwa aktualnie zalogowanego uytkownika.
	 */
	void initPropetiesDlgByUrl( const UrlInfoExt & , const QString & loggedUserName );


	/** Metoda powoduje usunicie (aktualnie przetwarzanego pliku/katalogu)
	 z listy zaznaczonych.
	 */
	void removeItemFromSIL();


	/** Metoda tworzy obiekt podgldu (klasy FileView) oraz czy sygnay ze
	 slotami.
	 */
	void createFileView();

	/** Ustawia wskanik do obiektu skojae plikw.\n
	 @param fa - wskanik do obiektu skojae plikw.
	 */
	void setFilesAssociation( FilesAssociation *fa ) { mFilesAssociation = fa; }



	/** Wcza bd wycza bezpieczne usuwnie plikw.
	 */
	virtual void setRemoveToTrash( bool ) {}

	/** Wcza albo wycza czynno waenia plikw i katalogw przed operacj
	 plikow (np. kopiowania).
	 */
	virtual void setWeighBeforeOperation( bool ) {}

	/** Wcza bd wycza bezpieczne usuwnie plikw.
	 */
	virtual void setTrashPath( const QString & ) {}

	/** Ustawia ciek do katalogu roboczego.\n
	 Uywane tylko w systemie ArchiveFS.
	 */
	virtual void setWorkDirectoryPath( const QString & ) {}

	/** Wcza albo wycza utrzymywanie poczenia ze zdalnym hostem.\n
	 Uywane tylko w systemie FtpFS.
	 */
	virtual void setStandByConnection( bool ) {}

	/** Ustawia ilo powtarzania wykonania operacji poczanie, gdy zdalny host
	 nie odpowiada.\n
	 Uywane tylko w systemach FtpFS i SambaFS.
	 */
	virtual void setNumOfTimeToRetryIfServerBusy( uint ) {}


	/** Ustawia status nadpisywania plikw w przypadku operacji kopiowania
	 i przenoszenia.\n
	 @param alwaysOverwite - rwne TRUE zaznacza na dialogu, opcje nadpisywania,
	 dla FALSE opcja ta nie jest zaznaczana.\n
	 Ustawienie to jest uywane przez dialog kopiowania.
	 */
	void setAlwaysOverwrite( bool alwaysOverwite ) { mAlwaysOverwrite = alwaysOverwite; }

	/** Ustawia status zachowywania praw dostpu w przypadku operacji kopiowania
	 i przenoszenia.\n
	 @param savePerm - rwne TRUE zaznacza na dialogu, opcje zachowania praw,
	 dla FALSE opcja ta nie jest zaznaczana.\n
	 Ustawienie to jest uywane przez dialog kopiowania. Domylnie waciwo ta
	 jest wczona.
	 */
	void setAlwaysSavePermission( bool savePerm ) { mSavePermision = savePerm; }

	/** Wcza lub wycza pokazywanie okienka ostrzegawczego przed operacj
	 usuwania plikw.\n
	 @param askBeforeDelete - rwne TRUE, wymusza pokazanie dialogu, dla FALSE
	  okienko nie zostanie pokazane.\n
	 Domylnie waciwo ta jest wczona.
	 */
	void setAlwaysAskBeforeDeleting( bool askBeforeDelete ) { mAskBeforeDelete = askBeforeDelete; }

	/** Wcza lub wycza operacj wejcia do katalogu po jego utworzeniu.\n
	 @param getInToDir - rwne TRUE, wymusza wejcie do katalogu, dla FALSE
	 ono nie nastpi.\n
	 Domylnie waciwo ta jest wyczona.
	 */
	void setAlwaysGetInToDirectory( bool getInToDir ) { mGetInToDir = getInToDir; }

	/** Wcza/wycza chowanie dialogu postpu po zakoczeniu operacji.\n
	 @param closeAfterFinished - rwne TRUE, oznacza zamknicie dialogu,
	 w przypadku FALSE nie jest on zamykany.\n
	 Domylnie waciwo ta jest wyczona.
	 */
	void setCloseProgressDlgAfterFinished( bool closeAfterFinished ) { sCloseProgressDlgAfterFinished = closeAfterFinished; }

private:
	ListViewExt *mListViewExt;
	ProgressDialog *mProgressDlg;
	PropertiesDialog *mPropertiesDialog;
	FindFileDialog *mFindFileDialog;
	FilesAssociation *mFilesAssociation;

	QString mTargetURL;
	QString mCurrentURL, mNewURL;
	QString mPreviousDirName;

	KindOfFilesSystem mCurrentFilesSystem;

	QStringList mSelectedFilesList;
	bool mSavePermision, mAlwaysOverwrite, mHardLink, mAskBeforeDelete, mGetInToDir;
	bool mOperationHasFinished;

	int mItemIdForOpenedFile;
	bool mOpenedFileEditMode;

	QProcess mProcess;


	/** Metoda powoduje uruchomienie zewntrznych programw dla podgldu kadego
	z podanej listy plikw.
	@em filesToView - lista nazw plikw do podgldu.\n
	Nazwy programw - przegldarek pobierane s z bazy skojarze plikw.
	*/
	void runExternalViewer( const QStringList & filesToView );

protected:
	/** Metoda suca do otwierania podanego pliku do podgldu.
	 */
	virtual void open( const QString & ) {}

	/** Metoda sprawdza czy podany plik istnieje i czy mona go odczytywa.
	 */
	virtual bool fileIsReadable( const QString & , bool ) { return TRUE; }

	/** Metoda uruchamia listowanie plikw z podanego katalogu.
	 */
	virtual void readDir( const QString & )  {}

	/** Zwraca liczb wszystkich plikw i katalogw w biecym katalogu.\n
	 @return suma plikw i katalogw z biecego.
	 */
	virtual uint numOfAllFiles()  { return 0; }

	/** Metoda uruchamia procedur utworzenia nowego pliku lub katalogu.
	 */
	virtual void createAnEmptyFile( const QString & ) {}

	/** Uruchamia kopiowanie lub przenoszenie plikw z podanej listy do podanego
	 katalogu.
	 */
	virtual void copyFiles( QString & , const QStringList & , bool , bool , bool , bool ) {}

	/** Metoda uruchamia usuwanie plikw z podanej listy.
	 */
	virtual void deleteFiles( const QStringList & ) {}

	/** Metoda uruchamia tworzenie linku lub grupy linkw, albo edycj
	 pojedynczego.
	 */
	virtual void makeLink( const QStringList & , const QStringList & , bool , bool , bool ) {}

	/** Metoda uruchamia ustawianie atrybutw dla podanych na licie plikw.
	 */
	virtual void setAttributs( const QStringList & , const UrlInfoExt & , bool , int ) {}

	/** Metoda wywoywana po uruchomieniu wyszukiwania na dialogu szukania pliku.
	 */
	virtual void startFindFile( const FindCriterion & ) {}

	/** Zwraca kod biecej operacji, patrz: @see Operation.\n
	 @return kod biecej operacji.
	 */
	virtual Operation currentOperation() const { return NoneOp; }

	/** Funkcja uruchamie procedur przerwania biecej operacji.
	 */
	virtual void breakCurrentOperation() {}


	/** Zwraca informacje o tym czy podany plik jest linkiem symbolicznym.\n
	 @return TRUE podany plik jest linkiem, FALSE - plik nie jest linkiem.
	 */
	virtual bool isLink( const QString & ) { return FALSE; }

	/** Zwraca ciek docelow dla podanego linku.\n
	 @return cieka docelowa linku.
	 */
	virtual QString readLink( const QString & ) { return QString::null; }


	/** Zwraca nazw aktualnie zalogowanego uytkownika.\n
	 @return nazwa uytkownika.
	 */
	virtual QString loggedOwner() { return QString::null; }

	/** Zwraca nazw grupy dla aktualnie zalogowanego uytkownika.\n
	 @return nazwa grupy.
	 */
	virtual QString loggedGroup() { return QString::null; }


	/** Funkcja powoduje wystartowanie lub zatrzymanie stopera na dialogu postpu.\n
	 @param start - rwne TRUE nakazuje uruchomienie stopera, FALSE powoduje jego
	 zatrzymanie.
	 */
	void startProgressDlgTimer( bool start );

	/** Zwraca nazw aktualnie wybranego pliku wraz ze ciek absolutn.\n
	 @return nazwa biecego pliku.
	 */
	QString currentFileName() { return mCurrentURL+mListViewExt->currentItemText(); }


	/** Zamienia prawa dostpu podane w postaci cigu 'rwx' lub wartoci
	 semkowej, na warto typu 'int'.\n
	 @param permissions - adres zmiennej, w ktrej naley umieci wynik,
	 @param oldPermStr - pierwotny cig opisujcy prawa dostpu,
	 @param newPermStr - nowy cig opisujcy prawa dostpu.
	 */
	void decodePermissions( int & permissions, const QString & oldPermStr, const QString & newPermStr );

	/** Funkcja powoduje zdekodowanie cigu tekstowego 'data czas' podanego przez
	 uytkownika podczas tzw. szybkiej zmiany atrybutu.\n
	 @param dateTime - adres obiektu z informacj o czasie i dacie, w ktrym jest
	 zapisywana zdekodowana nowa warto daty i czasu,
	 @param oldTime - stara warto daty i czasu,
	 @param timeStr - nowa warto daty i czasu podana przez uytkownika.\n
	 Cig tekstowy podany jako @em timeStr powinien mie nastpujcy format:
	 yyyy-mm-dd hh-mm-ss, przy czym nie wszystkie pola musz tu wystapi -
	 - konieczne jest co najmniej jedno z daty lub czasu.
	 */
	void decodeTime( QDateTime & time, const QDateTime & oldTime, const QString & newTimeStr );

public slots:
	/** Slot powoduje zmian biecego katalogu o jeden poziom wyej.
	 */
	void slotCloseCurrentDir();

	/** Slot uruchamia procedure otwarcia katalogu, pliku zwykego, lub archiwum.\n
	 @param url - nazwa pliku lub katalogu.\n
	 Jeli nastpi prba wyjcia z najniszego katalogu w systemie plikw innym
	 ni lokalny, wtedy wymuszane jest przejcie do poprzedniego katalogu z
	 lokalnego systemu plikw.\n \n Uwaga:\n
	 Jeli ostatni znak w podanej nazwie jest znakiem slash ('/'), wtedy nastpuje
	 prba otwarcia katalogu o takiej nazwie. Jako nazw mona rwnie poda
	 nastpujce skrty: '..', '~'.
	 */
	void slotOpen( const QString & url );


	/** Slot - pomocnik (porednik) listowania plikw z systemu plikw.\n
	 @param urlInfo - adres obiektu z informacj o pliku.\n
	 Jest on wywoywany przez dziedziczce po tej klasie, obiekty klas systemu
	 plikw (LocalFS, FtpFS, etc.), i powoduje wstawianie elementu na widok listy.
	 */
	void slotListInfo( const UrlInfoExt & url );

	/** W slocie ustawiana jest tylko skadowa zawierajca biec ciek
	 systemu plikw.\n
	 @param newPath - nowa cieka, ktr naley ustawi jako biec.\n
	 Jest on wykorzystywany w widoku drzewa przez obiekt klasy widoku
	 listy (przy poruszaniu kursorem moe si zmieni biecy katalog).
	 */
	void slotSetCurrentURL( const QString & newPath );

	/** Slot powoduje pobranie informacji o pliku wprost z widoku listy.\n
	 @param fileName - nazwa pliku, ktremu naley pobra reszte informacji,
	 @param urlInfo - adres obiektu, w ktrym umieszczana jest pobrana informacja.
	 */
	void slotGetUrlInfoFromList( const QString & fileName, UrlInfoExt & );


	/** Slot powoduje pokazanie dialogu z pytaniem o nadpisywanie docelowego
	 pliku.\n
	 @param sourceFileName - nazwa rdowego plik,
	 @param targetFileName - adres obiektu zawierajcego nazw docelowego pliku,
	 przy czym slot moe tu wpisa now nazw (jeli uytkownik wybierze jej
	 zmian),
	 @param result - adres zmiennej, ktra bdzie zawiera kod wcinitego guzika.\n
	 Dostpne s nastpujce odpowiedzi: 'Yes', 'No', 'If different size', 'Rename',
	 'All', 'Update all', 'None', 'Cancel'. Przed pokazaniem dialogu wstrzymywany
	 jest stoper na dialogu postpu, a po wybraniu opcji stoper jest uruchamiany
	 ponownie.
	 */
	void slotShowFileOverwriteDlg( const QString & sourceFileName, QString & targetFileName, int & result );

	/** Slot powoduje pokazanie dialogu z pytaniem czy podany katalog usun
	 rekurencyjnie, moliwymi odpowiedziami s: 'Yes', 'No', 'All', 'None',
	 'Cancel'.\n
	 @param dirName - nazwa usuwanego katalogu,
	 @param result - adres zmiennej, ktra bdzie zawiera kod wcinitego guzika.\n
	 Przed pokazaniem dialogu wstrzymywany jest stoper na dialogu postpu, a po
	 wybraniu opcji stoper jest uruchamiany ponownie.
	 */
	void slotShowDirNotEmptyDlg( const QString & dirName, int & result );


	/** W tym slocie nastpuje uaktualnienie informacji o wyszukiwaniu pliku na
	 dialogu wyszukiwania.\n
	 @param matches - ilo pasujcych elementw,
	 @param files - ilo wszystkich przejanych plikw,
	 @param directories - ilo wszystkich przejanych katalogw.
	 */
	void slotUpdateFindStatus( uint matches, uint files, uint directories );

	/** Slot uruchamia procedur wyszukiwania pliku.\n
	 @param findCriterion - adres obiektu zawierajcego kryterium wyszukiwania.\n
	 Jeli kryterium jest puste, wtedy dialog wyszukiwania jest przestawiany w
	 stan 'zakoczono wyszukiwanie'.
	 */
	void slotStartFind( const FindCriterion & findCriterion );

	/** Slot powoduje wstawienie na list dialogu wyszukiwania pliku lub wzorca,
	 podanego elemenu.\n
	 @param urlInfo - adres obiektu zawierajcego informacj o pliku.
	 */
	void slotInsertMatchedItem( const UrlInfoExt & url );

	/** Slot powoduje wczytywanie pliku wybranego do podgldu.
	 */
	virtual void slotReadFileToView( const QString & , QByteArray & , int &, int ) {}

	/** Slot powoduje zainicjowanie podanego obiektu z informacj o pliku
	 atrybutami podanego.
	 */
	virtual void slotGetUrlInfo( const QString & , UrlInfoExt & ) {}

	/** Slot uruchamia procedur wstrzymania lub wznowienia biecej operacji.
	 */
	virtual void slotPause( bool ) {}

private slots:
	/** Slot urchamia procedur zmiany atrubutw dla zaznaczonych plikw.\n
	@param newUrlInfo - adres obiektu zawierajcego now informacj o pliku,
	@param allSelected - rwne TRUE, wymsza zmian atrybutw dla wszystkich
	zaznaczonych plikw, w przeciwnym razie zmiana jest wykonywana tylko dla
	biecego.\n
	Slot jest wywoywany (przy pomocy sygnau) przez dialog waciwoci pliku.
	Patrz te: @see setAttributs().
	*/
	void slotChangePropertiesOfFile( const UrlInfoExt & url, bool allSelected );

	/** Slot wywoywany w momencie zaznaczenia checkBox-u opisanego jako
	 "Close after finished", na dialogu postpu danej operacji.\n
	 @param closeState - rwne TRUE oznacza zamknicie dialogu postpu.
	 */
	void slotCloseAfterFinished( bool closeState );

	/** Slot uruchamia procedur waenia katalogu.\n
	 @param itemNum - indeks elementu na licie nazw plikw.
	 Slot jest wywoywany (przy pomocy sygnau) przez dialog waciwoci pliku.
	 Waenie plikw jest uruchamiane w tzw. trybie 'realWeigh', po wicej
	 informacji patrz: @see sizeFiles().
	 */
	void slotCountingDirectory( int itemNum );

	/**
	 */
	void slotOperationInBackground();

	/** Slot powoduje usunicie dialogu waciwoci pliku oraz przerwanie biecej
	 operacji.
	 */
	void slotClosePropertiesDlg();

	/** Slot powoduje pokazanie dialog z pytaniem o potwierdzenie przerwania
	 biecej operacji.\n
	 Po potwierdzeniu przerwania wywoywana jest funkcja zatrzymujca operacj.
	 Slot jest wywoywany w momencie przerwania jakiejkolwiek operacji.
	 */
	void slotCancelOperation();


	/** Slot powoduje usunicie dialogu wyszukiwania pliku oraz przerwanie
	 biecej operacji.
	 */
	void slotCloseFindFileDlg();

	/** W slocie uruchamiana jest procedura usunicia podanego pliku.\n
	 @param fileName - nazwa pliku, ktry naley usun.\n
	 Slot uywany przez obiekt klasy FindFileDialog.
	 */
	void slotDeleteFile( const QString & fileName );

	/** Slot powoduje ewentualn zmian biecego katalogu oraz skok kursora
	 na licie do podanego pliku.\n
	 @param fileName - nazwa pliku, ktry naley otworzy.\n
	 Jest on uywany przez obiekt klasy FindFileDialog.
	 */
	void slotJumpToTheFile( const QString & fileName );

	/** Slot powoduje otwarcie podanego pliku w podanym trybie.\n
	 @param fileName - nazwa pliku do otwarcia,
	 @param editMode - tryb otwarcia (TRUE - edycja, FALSE - podgld).\n
	 Slot uywany jest przez obiekt klasy FindFileDialog.
	 */
	void slotViewFile( const QString & fileName, bool editMode );

signals:
	/** Sygna powoduje pokazanie na statusie panela podanej informacji.\n
	 @param info - tekst do wywietlenia.\n
	 Uywany jest do pokazywania zmiany statusu w biecej operacji,
	 np. poczenia ze zdalnym hostem.
	 */
	void signalSetInfo( const QString & info );

	/** Sygna oznacza zakoczenie operacji.
	 @param operation - zakoczona operacja,
	 @param noError - rwne TRUE oznacza, e operacja zakoczya si bezbdnie,
	  FALSE mwi o tym, e wystpi bd.\n
	 Jest wysyany po zakoczenia operacji plikowej.
	 */
	void signalResultOperation( Operation operation, bool noError );


	/** Sygna uywany do pokazywania postpu dla operacji na pliku,
	 np. kopiowania.\n
	 @param done - cakowita liczba przetransportowanych bajtw z danego pliku,
	 @param total - liczba wszystkich bajtw danego pliku,
	 @param bytesTransfered - mieci liczb bajtw aktualnie przetransportowanych.
	 */
	void signalDataTransferProgress( long long done, long long total, uint bytesTransfered );

	/** Sygna uywany do zliczania plikw podczas wykonywania biecej operacji.\n
	 @param value - liczba plikw,
	 @param fileCounting - warto TRUE oznacza, e zliczane s pliki,
	 w przeciwnym razie s to katalogi,
	 @param valueOfAll - waro TRUE oznacza, e zliczana jest ilo wszystkich
	 (uywane przy operacji waenia), dla FALSE - ilo przetworzonych.
	 */
	void signalFileCounterProgress( int value, bool fileCounting, bool setTotalValue );

	/** Sygna uywany do wysyania nazwy aktualnie przetwarzanego pliku.\n
	 @param fileName - nazwa przetwarzanego pliku.
	 */
	void signalNameOfProcessedFile( const QString & fileName );

	/** Sygna uywany do ustawiania nazwy operacji na dialogu postpu.\n
	 @param op - kod operacji (patrz @see Operation).
	 */
	void signalSetOperation( Operation op );

	/** Sygna wysya warto procentow informujc o postpie caej operacji.\n
	 @param done - warto w procentach oznaczajca ile dotd zrobiono,
	 @param totalWeight - waga wszystkich przetwarznych plikw.
	 */
	void signalTotalProgress( int done, long long totalWeight ); // done in percent


	/** Sygna nakazuje otwiercie podanego archiwum.\n
	 @param fileName - nazwa archiwum.
	 */
	void signalOpenArchive( const QString & fileName );


	/** Sygna uruchamia wczytywanie pliku do podgldu.
	 */
	void signalReadyRead();

};

#endif
